#!/usr/bin/php
<?php
// 04/10/2017 acress - created
// 04/11/2017 acress - mark already granted as Success
// 05/04/2017 acress - fixes for error conditions in UI testing,
//                     restructured to use class RemoteGrant instead

/* Requires for standalone execution. */
require_once('/var/www/html/api/includes/bp.php');
require_once('/var/www/html/api/includes/logger.php');
require_once('/var/www/html/api/includes/function.lib.php');
require_once('/var/www/html/api/includes/constants.lib.php');

class RemoteGrant
{
    private $BP;

    public function __construct($BP = null)
    {
        $this->BP = $BP;
        $this->functions = new Functions($this->BP);
    }

    function remoteAuthenticate($targetName, $user = null, $pswd = null) {
        global $Log;
        $info['username'] = $user;
        $info['password'] = $pswd;
        $curl = curl_init();
        $url = "https://" . $targetName . "/api/login";
        $credentials = array("username" => $info['username'], "password" => $info['password']);
        $credentials_string = json_encode($credentials);
        $Log->writeVariable("authenticating, url is $url");
        $Log->writeVariableDBG("authenticating, auth is $credentials_string");
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($curl, CURLOPT_POSTFIELDS, $credentials_string);
        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json')
        );
        $result = curl_exec($curl);
        if ($result == false) {
            $Log->writeVariable("the curl request to authenticate failed: ");
            $Log->writeVariable(curl_error($curl));
            $result = "Attempt to connect to remote appliance failed.  Please ensure the appliance is powered on and its network address is resolvable.";
        } else {
            // return as a string
            $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            $Log->writeVariable('curl request: http code');
            $Log->writeVariable($httpcode);
            $result = json_decode($result, true);
            if ($httpcode == 201) {
                // Successfully logged into the remote appliance.
                // Set the results, and cache the new token
                // The save cache API can fail if it not proper credentials
                $result['remote_token'] = $result['auth_token'];
                $auth_token = $result['auth_token'];
                $source_id  = $result['source_id'];
		// print "auth_token = $auth_token\n"; // for debug
                $ret = $this->BP->save_target_token($targetName, $auth_token, $source_id);
                // set cached status which will dictate to the caller if a logout is needed
                if ($ret === true) {
                    $result['cached'] = true;
                }
                else {
                    $result['cached'] = false;
                }
            } else {
                $result = $result['result'];
                if (is_array($result[0]) && isset($result[0]['message'])) {
                    $result = $result[0]['message'];
                } else {
                    $result = "Attempt to connect to remote appliance failed (error code: " . $httpcode . ").";
                }
            }
        }
        curl_close($curl);
        return $result;
    }

    function run_login($ip = null, $username = null, $passwd = null) {
	$auth = null;
	$url = "https://$ip/recoveryconsole/bpl/login.php?name=$username&password=$passwd";
	$login = file_get_contents($url);
	if ($login === false) {
		$login = file_get_contents($url);
	}
	if ($login === false) {
		print "Error: failed login\n";  //debug
		return $auth;
	}
	// var_dump($login); // for debug
	if (isset($login['AuthenticationString'])) {
		$loginobj = new SimpleXMLElement($login);
		$auth = $loginobj->AuthenticationString;
	} else {
		print "Error: failed login\n";  //debug
	}
	return $auth;
    }

    function remote_request($ip, $request, $api, $parameters, $data = NULL, $sid = false, $auth_token = NULL, $force_authenticate = true, $returnRaw = false) {
        global $Log;
        $need_to_authenticate = false; // also dictates logouts per caching results.
        $retried_failed_login = false;
        $failed_login_message = "Error: Your session has expired or is invalid";
        $source_id = false;
        $host = $ip;
        if ($ip === "") {
            $errorMessage = "No IP specified or target host found, so no remote request will run.";
            $Log->writeVariable("remote request: $errorMessage");
            return $errorMessage;
        }
        // auth token and sid provided? If not, check the cache.
        if ($auth_token === NULL || $sid === false) {
            $cachedInfo = $this->functions->getLocallyCachedTargetInfo($target);
            if (is_array($cachedInfo) && isset($cachedInfo['auth_token']) && isset($cachedInfo['source_id'])) {
                $auth_token = $cachedInfo['auth_token'];
                $source_id = $cachedInfo['source_id'];
            }
        } else {
            // if an auth_token was provided, the incoming sid must be remote 
            $Log->writeVariable("remote request: provided sid $sid being used as the remote source id too.");
            $source_id = $sid;
        }

        if ($auth_token === NULL || $source_id === false) {
            return "You must first connect to the target appliance in order to run this command";
        }
        if ($sid === false) {
            $sid = $source_id;
        }

        if (1) {
            // After authentication, run the request.
            $curl = curl_init();
            $base_url = "https://" . $ip;
            $url = $base_url . $api;
            $separator = "?";
            if ($sid !== NULL) {
                $url .= $separator . "sid=" . $sid;
                $separator = "&";
            }
            if ($parameters !== "") {
                $url .= $separator . $parameters;
                $separator = "&";
            }
            if (isset($source_id) && $source_id !== '') {
                $url .= $separator . "source_id=" . $source_id;
                $separator = "&";
            }

            $url = str_replace(' ', '%20', $url);
            $url = str_replace('+', '%2B', $url);
            curl_setopt($curl, CURLOPT_URL, $url);
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $request);
            if ($request === 'PUT' || $request === 'POST') {
                if ($data !== NULL) {
                    $payload = json_encode($data);
                    curl_setopt($curl, CURLOPT_POSTFIELDS, $payload);
                }
            };
            curl_setopt($curl, CURLOPT_HTTPHEADER, array(
                    'Content-Type: application/json',
                    'AuthToken: ' . $auth_token )
            );
            $result = curl_exec($curl);
            if ($result == false) {
                $Log->writeVariable("the curl request failed: ");
                $Log->writeVariable(curl_error($curl));
                $message = "An unexpected failure occurred while running a request between the source and target. Please try your request again.\n\n
                If this continues to fail, please examine the latest /usr/bp/logs.dir/api_*.log files on the source appliance for more details.";
                $result = array();
                $result['error'] = 500;
                $result['message'] = $message;
            } else {
                // return as a string
                $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
                $Log->writeVariable('curl request: http code');
                $Log->writeVariable($httpcode);
                if (!$returnRaw || ($returnRaw && $httpcode !== 201 && $httpcode !== 200)) {
                    $result = json_decode($result, true);
                }
                if (($request == "POST" && $httpcode == 201) ||
                    (($request == "GET" || $request == "PUT" || $request == "DELETE") && $httpcode == 200)
                ) {
                    // Okay (200) for GET, PUT, DELETE.  Created (201) for POST.
                    if (!$returnRaw) {
                        if (is_array($result) && $need_to_authenticate == true) {
                            // include the token if we had to authenticate.
                            $result['remote_token'] = $auth_token;
                        }
                        if (is_array($result)) {
                            $result['target_name'] = $ip;
                        }
                    }
                } else {
                    $result = $result['result'];
                    if (is_array($result[0]) && isset($result[0]['message'])) {
                        $message = $result[0]['message'];
                        if ($retried_failed_login == false) {
                            // Check for expired tokens results, and retry using a new token
                            if (strstr($message, $failed_login_message)) {
                                $retried_failed_login = true;
                                $auth_token = NULL;
                            }
                        }
                    } else {
                        $message = "Attempt to run remote curl command failed (error code: " . $httpcode . ").";
                    }
                    $result = array();
                    $result['error'] = 500;
                    $result['message'] = $message;
                }
            }
            curl_close($curl);

            if ($need_to_authenticate && isset($auth_token)) {
                // Note: the ip value here is most likely a host name
        	$this->functions->remoteLogout($ip, $auth_token);
            }
            return $result;
        } else {
            $result = array();
            $result['error'] = 500;
            $result['message'] = "Insufficient privileges";
            return $result;
        }
    }

    function run_logout($ip = null, $token = null) {
        $data = array("cookie" => $token);
        $result = $this->remote_request($ip, "POST", "/api/logout", "", $data, NULL, $token);
        return $result;
    }

    function run_remote_grant($ip = null, $auth = null, $host = null) {
	$msg = null;
	$url = "https://$ip/recoveryconsole/bpl/managers.php?type=grant&lvl=5&hostname=$host&auth=$auth";
	$ret = file_get_contents($url);
	if ($ret === false) {
		$msg = "Error: connect failed";
		return $msg;
	}
	if (isset($ret['Result'])) {
		$retobj = new SimpleXMLElement($ret);
		$res = $retobj->Result;
		if ( $res == 1 ) {
			$msg = "Success";
		} else {
			$rmsg = $retobj->ErrorString;
			if (strpos($rmsg, 'already been granted') !== false) {
				$msg = "Success, already granted";
			} else {
				$msg = "Error: $rmsg";
				// print "result=$res $msg\n"; //debug
				// var_dump($rmsg);   //debug
			}
		}
	} else {
		$msg = "Error: failed grant";
	}
	return $msg;
    }
}

if (isset($argv[1])) { $remip = $argv[1]; }
if (isset($argv[2])) { $user = $argv[2]; }
if (isset($argv[3])) { $pswd = $argv[3]; }
if (isset($argv[4])) { $myhost = $argv[4]; } 
else  {
	print "Usage:  rungrant.php <dest ip> <user> <password> <hostname>\n";
	exit(-1);
}

$Log = new Logger();
$BP = new BP();
$grant = new RemoteGrant($BP);
// $auth = $grant->run_login($remip, $user, $pswd); 
$result = $grant->remoteAuthenticate($remip, $user, $pswd); 
if ($result == null) {
	print "Error: failed login\n";
	exit(-1);
}
// var_dump($result); // for debug
if (is_array($result) && isset($result['auth_token'])) {
	$auth = $result['auth_token'];
} else {
	print "Error: failed login $result\n";
	exit(-1);
}

print "Attempting to grant $myhost on $remip ... \n";
$resmsg = $grant->run_remote_grant($remip, $auth, $myhost); 
$success = 1;
if ($resmsg !== null) {
	if (strpos($resmsg, 'Success') !== false) {
		$success = 0;
	}
}
print "return=$success, $resmsg\n";

$result = $grant->run_logout($remip, $auth); 
// var_dump($result); // for debug
$logout = 1;
if (is_array($result) && isset($result['result'])) {
    $logout = 0; // $result['result'][0]['code'];
}
if ( $logout !== 0 ) {
   print "logout error = ".$result['message']."\n";
}

exit($success);
?>
