#!/usr/bin/php
<?php
	// Operator constants 
	define("EQUAL", 0);
	define("NOT_EQUAL", 1);
	define("CONTAINS", 2);
	define("NOT_CONTAINS", 3);
	define("STARTS_WITH",4);
	define("ENDS_WITH", 5);

	// Container constants 
	define("DATACENTER", 4);
	define("ESX_CLUSTER", 5);
	define("ESX_SERVER", 6);
	define("DATASTORE", 7);
	define("RESOURCE_POOL", 8);
	define("VAPP", 9);
	define("FOLDER", 10);
	define("VM_NAME", 11);
	
	// Container Type constants 
	define("DATACENTER_TYPE", "Datacenter");
	define("ESX_CLUSTER_TYPE", "ClusterComputeResource");
	define("ESX_SERVER_TYPE", "HostSystem");
	define("DATASTORE_TYPE", "Datastore");
	define("RESOURCE_POOL_TYPE", "ResourcePool");
	define("VAPP_TYPE", "VirtualApp");
	define("FOLDER_TYPE", "Folder");
	
	// Lookup Table
	$LOOKUP_CONTAINER = array();	
	$LOOKUP_CONTAINER_VM = array();
	$LOOKUP_ESX_VM = array();
	$log = "";
	$log_count = 25;
	$program_name   = basename($argv[0], ".php");	
	#---------------------------------------------------------------------------
	# Log helper functions
	#---------------------------------------------------------------------------
	function setup_logging()
	{
  		global $program_name;
  		global $log;
  		global $log_count;

 		# turn off php errors while we rename files that may not exist
		error_reporting(0);

		$logname = $program_name;
		$i = $log_count;
		while ($i >= 1) {
			$m = $i + 1;
			rename("/usr/bp/logs.dir/$logname.log.$i", "/usr/bp/logs.dir/$logname.log.$m");
			$i--;
		}
		rename("/usr/bp/logs.dir/$logname.log", "/usr/bp/logs.dir/$logname.log.1");

		# set basic php reporting
		error_reporting(E_ERROR | E_WARNING | E_PARSE);

		$log = fopen("/usr/bp/logs.dir/$logname.log", "w");
		if ($log == false) {
			print "Could not open log file for writing.\n";
			exit(1);
		}

		$msg = "### $program_name started on ".date('Y-m-d H:i:s');
		fprintf($log, "%s\n", $msg);
	}

	#-----------------------------------------------------------------------------
	# function check_for_vprotect_error($vprotect_xml) 
	# @desc: This function checks if thers is an error xml from vprotect. 
	# 
        # @vprotect_xml:	xml object returned by vprotect.
	# returns:		Boolean true/flase.     
	#-----------------------------------------------------------------------------
	function check_for_vprotect_error($vprotect_xml) {
		$ret = true;
		foreach($vprotect_xml->children() as $child) {
			$c = $child->getName(); // vprotect_error;
			if ($c != "vprotect_error") {
				$ret = true;
				break;
			} else {
				$ret = false;
			} 
		}
		return $ret;
	}

	#-----------------------------------------------------------------------------
	# function delete_vms_from_schedule($dbconn, $instance_ids) {
	# @desc: This function removes list of VM from their respective schedules 
	# 
        # @dbconn:		database connection handle.
	# @instance_ids:	list of instance ids.
	# returns:		true (always).     
	#-----------------------------------------------------------------------------
	function delete_vms_from_schedule($dbconn, $instance_ids) {
		global $log;

		fprintf($log, "delete_vms_from_schedule:\n");
		if (empty($instance_ids)){ 
			fprintf($log, "Null Input\n");
			return true;
		}
		$instance_ids_comma_list = implode(",", $instance_ids); 
		fprintf($log, "Instance_ids: $instance_ids_comma_list\n");
		foreach($instance_ids as $instance_id) {
			$rem_query = "DELETE FROM bp.schedule_client_assoc";
			$rem_query .= " WHERE instance_id = '$instance_id'";	
			$rem_res = pg_query($dbconn, $rem_query);
			if ($rem_res == false) {
				fprintf($log, "Failed to delete $instance_id\n");
				continue;
			}
		}
		return true;
	}

	#-----------------------------------------------------------------------------
	# function get_vms_instance_id($dbconn, $uuid_list) {
	# @desc: This function returns list of VM instance ids corresponding to uuids  
	# 
        # @dbconn:	database connection handle.
	# @uuid_list:	VM's instance uuid list.
	# returns:	List of VM's instance_ids.     
	#-----------------------------------------------------------------------------
	function get_vms_instance_id($dbconn, $uuid_list) {
		//$uuid_comma_list = implode(",", $uuid_list); 
		global $log;

		$vm_cnt = 0;
		$vm_instance_list = array();
		fprintf($log, "get_vms_instance_id:\n");
		if ($uuid_list == NULL) { 	
			fprintf($log, "Null Input\n");
			return $vm_instnce_list;
		}
		foreach($uuid_list as $vm_uuid) {
			$vm_query = "SELECT instance_id";
			$vm_query .= " FROM bp.vmware_vms";
			$vm_query .= " WHERE vm_instance_uuid = '$vm_uuid'";

			$vm_res = pg_query($dbconn, $vm_query);
			if ($vm_res == false) {
				fprintf(log, "Failed to find instance id for $vm_uuid\n");
				continue;
			}
			if (pg_num_rows($vm_res) == 0) {
				fprintf($log, "Skip uuid = $vm_uuid\n");
				continue;
			} 
			$vm_query_row = pg_fetch_assoc($vm_res, 0);
			$vm_instance_list[$vm_cnt] = $vm_query_row["instance_id"];
			$vm_cnt++;
		}
		return $vm_instance_list;
	}

	#-----------------------------------------------------------------------------
	# function get_vms_for_schedule($dbconn, $schedule_id)
	# @desc: This function returns list of VMs for a given schedule_id.
	# 
        # @dbconn:	database connection handle
	# @schedule_id:	schedule identifier.
	# returns:	List of VMs.     
	#-----------------------------------------------------------------------------
	function get_vms_for_schedule($dbconn, $schedule_id) {
  		global $log;

		fprintf($log, "get_vms_for_schedule: $schedule_id\n");
		$sched_query = "SELECT instance_id FROM bp.schedule_client_assoc";
		$sched_query .= " WHERE schedule_id = '$schedule_id'";
		$sched_res = pg_query($dbconn, $sched_query);
		if ($sched_res == false) {
			fprintf($log, "Failed to query schedule_client_assoc\n");
			return NULL;
		}
        	$sched_query_nrows =  pg_num_rows($sched_res);
		if (pg_num_rows($sched_res) == 0) { 
			fprintf($log, "No vms in the schedule\n");
			return NULL;
		}
		$vm_list = array();
		for ($s_cnt = 0; $s_cnt < $sched_query_nrows; $s_cnt++) {
			$sched_query_row = pg_fetch_assoc($sched_res, $s_cnt);
			$vm_list[$s_cnt] = $sched_query_row["instance_id"];
		}
		return $vm_list;
	}

	#-----------------------------------------------------------------------------
	# function get_vms_for_container($container, $operator, $pattern, $address,
	# 	                           $login, $password) 
	# @desc: This fuction takes 'match critera' &  'match string' elements of 
	#	 a regular expression, and login credentials for a vCenter/ESX server.
	#        It creates a list of VM's instance_ids as per the input list of reg- 
	#        expressions. The list of Containers and corresponding VMs in this is 
	#        fetched straight from the vCenter/ESX servers using Unitrends utility:
	#	 /usr/bp/bp/bin/vprotect. 
	#
	# @container:	container constant from the #defines above. (e.g. ESX_SERVER etc) 
	# @operator:	operator constant from the #defines above. (e.g. EQUAL etc) 
	# @pattern:	'match-string' to be used for matching criteria 
	# @address:	address of ESX/vCenter server (as stored in bp.vmware tables) 
	# @login: 	username (as stored in bp.vmware tables) 
	# @password: 	password (as stored in bp.vmware tables)
	# returns:	List of VMs.     
	#-----------------------------------------------------------------------------
	function get_vms_for_container($container_type, $container, $operator, $pattern, $address, 
	                                $login, $password) {
		
		global $log;
		global  $LOOKUP_CONTAINER;
		global  $LOOKUP_CONTAINER_VM;
		fprintf($log, "get_vms_for_container(esx=$address, ct=$container_type, c=$container, op=$operator, p=$pattern)\n");	
		
		$null_array = array();
		if (!isset($LOOKUP_CONTAINER["$address"]["$container_type"])) {
			fprintf($log, "Reading Container XML over Network [via Vprotect]\n");
			/* Command to get all container */
			$cmd_container = "/usr/bp/bin/vprotect/vprotect --server '$address'";
			$cmd_container .= " --username '$login' --password '$password' --bpcmd";
			$cmd_container .= " '$container' 2>/dev/null";

			$output_container = shell_exec($cmd_container);
			if (!is_string($output_container) || !strlen($output_container)) {
				fprintf($log, "Null XML returned.\n");
				return $null_array;
			}
			/* Check XML is well formed */
			$xml = simplexml_load_string($output_container);
			if (!$xml) {
				fprintf($log, "XML is not in wff.\n");
				return $null_array; 
			}
			/* Check for the vprotect error message  */
			$vprotect_res = check_for_vprotect_error($xml);
			if ($vprotect_res == false) {
				fprintf($log, "Vprotect returned Error\n");
				return $null_array; 
			}
			$LOOKUP_CONTAINER["$address"]["$container_type"] = $output_container;
		} else {
			fprintf($log, "Reading Container XML from the look-up table [in-memory cache]\n");
			$output_container = $LOOKUP_CONTAINER["$address"]["$container_type"];
		}

		$container_xml = new SimpleXMLElement($output_container);

		$i = 0;
		$matched_containers = array();
		foreach ($container_xml->vprotect_info->container as $container) {
			$container_name_str = (string) $container->name;
			switch($operator) {
				case EQUAL :
					if (strcasecmp($container_name_str, $pattern) == 0) {
						$matched_containers[$i] = $container_name_str;
						$i++; 
					}
				break;
				case NOT_EQUAL :
					if (strcasecmp($container_name_str, $pattern) != 0) {
						$matched_containers[$i] = $container_name_str;
						$i++; 
					}
				break;
				case CONTAINS :
					if (stripos($container_name_str, $pattern) !== false) {
						$matched_containers[$i] = $container_name_str;
						$i++; 
					}
				break;
				case NOT_CONTAINS :
					if (stripos($container_name_str, $pattern) == false) {
						$matched_containers[$i] = $container_name_str;
						$i++; 
					}
				break;
				case STARTS_WITH :
					if (strpos($container_name_str, $pattern) === 0) {
						$matched_containers[$i] = $container_name_str;
						$i++; 
					}
				break;
				case ENDS_WITH :
					$end= strlen($container_name_str) - strlen($pattern);
					if (strripos($container_name_str, $pattern, 0) === $end) {
						$matched_containers[$i] = $container_name_str;
						$i++; 
					}
				break;
				default :
					fprintf($log, "EORROR: Unsupported Regular Expression: $operator\n");
				break;
			}
		}
		if (!empty($matched_containers)) {
			$mc_comma_list = implode(",", $matched_containers);
			fprintf($log, "matched_container = ($mc_comma_list)\n");
		} else {
			fprintf($log, "matched_container = None\n");
		}
		
		// Find VMs for the matching containers. 
		$i = 0;
		$xml_vm_uuids = array();
		foreach ($matched_containers as $m_container) {
			if (!isset($LOOKUP_CONTAINER_VM["$address"]["$container_type"]["$m_container"])) {
				fprintf($log, "Reading VM XML over Network [via Vprotect]. CN=$m_container\n");
				$cmd_vm = "/usr/bp/bin/vprotect/vprotect --server '$address' --username";
				$cmd_vm .= " '$login' --password '$password' --containertype '$container_type'";
				$cmd_vm .= "  --containername '$m_container'";
				$output_vm = shell_exec($cmd_vm);
				if (!is_string($output_vm) || !strlen($output_vm)) {
					fprintf($log, "Null XML returned.\n");
					return $null_array;
				}
				$xml = simplexml_load_string($output_vm);
				if (!$xml) {
					fprintf($log, "XML returned is not wff.\n");
					return $null_array;
				}			 
				$vprotect_vm_res = check_for_vprotect_error($xml);
				if ($vprotect_vm_res == false) {
					fprintf($log, "Vprotect returned Error\n");	
					return $null_array;
				}
				$LOOKUP_CONTAINER_VM["$address"]["$container_type"]["$m_container"] = $output_vm; 
			} else {
				fprintf($log, "Reading Container VM XML from the look-up table [in-memory cache]. CN=$m_container\n");
				$output_vm = $LOOKUP_CONTAINER_VM["$address"]["$container_type"]["$m_container"]; 
			}
			$vm_xml = new SimpleXMLElement($output_vm);
			foreach ($vm_xml->vprotect_info->virtual_machines->virtual_machine as $vm) {
				$xml_vm_uuids[$i]   = (string) $vm->uuid;
				$i++;
			}	
		}
		fprintf($log, "Total VMs for Matching Containers: $i\n");
		return $xml_vm_uuids;
	}

	#-----------------------------------------------------------------------------
	# function get_vms_for_vmname($operator, $pattern, $address, $login, $password) 
	# @desc: This function takes 'match critera' & 'match pattern' elements of a 
	#        regular expressions, and login credentials for a vCenter/ESX server.
	#        It creates a list of VM instance_ids as per the input match-pattern 
	#        and criteria. The list of VMs here is fetched straight from the 
	#        vCenter/ESX servers using Unitrends utility :
	#	 /usr/bp/bp/bin/vprotect. 
	#
        # @dbconn:	database connection handle
	# @operator:	operator constant from the #defines above. (e.g. EQUAL etc) 
	# @pattern:	'match-string' to be used for matching criteria 
	# @address:	address of ESX/vCenter server (as stored in bp.vmware tables) 
	# @login: 	username (as stored in bp.vmware tables) 
	# @password: 	password (as stored in bp.vmware tables)
	#
	# returns:	List of VMs.     
	#-----------------------------------------------------------------------------
	function get_vms_for_vmname($operator, $pattern, $address, $login, $password) {
  		global $log;
		global $LOOKUP_ESX_VM;
		
		$null_array = array();
		fprintf($log, "get_vms_for_vmname(esx=$address, o=$operator, p=$pattern)\n");
		if (!isset($LOOKUP_ESX_VM["$address"])) { 
			fprintf($log, "Reading ESX VM XML from Network [via Vprotect].\n");
			$cmd_vms = "/usr/bp/bin/vprotect/vprotect --server '$address' --username";
			$cmd_vms .= " '$login' --password '$password' --bpcmd get_vm_info 2>/dev/null";

			$output_vms = shell_exec($cmd_vms);
			if (!is_string($output_vms) || !strlen($output_vms)) {
				fprintf($log, "Null XML returned.\n");
				return $null_array;
			}

			/* Check XML is well formed */
			$xml = simplexml_load_string($output_vms);
			if (!$xml) {
				fprintf($log, "XML returned is not wff.\n");
				return $null_array;
			}
			/* Check for the vprotect error message  */
			$vprotect_res = check_for_vprotect_error($xml);
			if ($vprotect_res == false) {
				fprintf($log, "Vprotect returned Error\n");	
				return $null_array;
			}
			$LOOKUP_ESX_VM["$address"] = $output_vms;
		} else {
			fprintf($log, "Reading ESX VM XML from the look-up table [in-memory cache].\n");
			$output_vms = $LOOKUP_ESX_VM["$address"];
		}

		$i = 0;
		$matched_vms = array();	
		$vms_xml = new SimpleXMLElement($output_vms);
		foreach ($vms_xml->vprotect_info->esx_server as $esx) {
			foreach ($esx->virtual_machines->virtual_machine as $virtual_machine) {
				$vm_name_str = (string) $virtual_machine->name;
				$vm_uuid_str = (string) $virtual_machine->instanceUuid;
				switch($operator) {
					case EQUAL :
						if (strcasecmp($vm_name_str, $pattern) == 0) {
							$matched_vms[$i] = $vm_uuid_str;
							$i++; 
						}
					break;
					case NOT_EQUAL :
						if (strcasecmp($vm_name_str, $pattern) != 0) {
							$matched_vms[$i] = $vm_uuid_str;
							$i++; 
						}
					break;
					case CONTAINS :
						if (stripos($vm_name_str, $pattern) !== false) {
							$matched_vms[$i] = $vm_uuid_str;
							$i++; 
						}
					break;
					case NOT_CONTAINS :
						if (stripos($vm_name_str, $pattern) == false) {
							$matched_vms[$i] = $vm_uuid_str;
							$i++; 
						}
					break;
					case STARTS_WITH :
						if (strpos($vm_name_str, $pattern) === 0) {
							$matched_vms[$i] = $vm_uuid_str;
							$i++; 
						}
					break;
					case ENDS_WITH :
						$end = strlen($vm_name_str) - strlen($pattern);
						if (strripos($vm_name_str, $pattern, 0) === $end) {
							$matched_vms[$i] = $vm_uuid_str;
							$i++; 
						}
					break;
					default :
						// bad entry in the database
						fprintf($log, "ERROR: Unsupported Regular Expression: $operator\n");
					break;
				}
			}
		}
		fprintf($log, "Total Matched VMs: $i\n");
		return $matched_vms;
	}

	#-----------------------------------------------------------------------------
	# function get_vms_for_regex_list($dbconn, $regex_list, $address, $login, 
	#            $password)
	# @desc: Fuctions takes in list of regular expressions, and login credentials
	#        for a vCenter/ESX server, and creates a list of VM's instance_ids as
	#        per the input list of reg expressions. The list of VMs here is fetech
	#        straight from the vCenter/ESX servers using Unitrends utility: 
	#        /usr/bp/bp/bin/vprotect. 
	#
        # @dbconn:	database connnection handle
	# @$regx_list:	list of regular expressions in following format:
	#	           [<container_1>;;<matching_criteria_1>;;<match_string_1>]* 
	#		   11;;0;;EAPP::7;;1;;DS-14TB
	# @address:	address of ESX/vCenter server 
	# @login:	username (from bp.vmware tables) 
	# @password:	password (from bp.vmware tables)    
	#-----------------------------------------------------------------------------
	function get_vms_for_regex_list($dbconn, $regex_list, $address, $login,
	                                 $password) {
  		global $log;

		fprintf($log, "get_vms_for_regex_list(reg_list=$regex_list, esx=$address)\n");
		
		$vms_for_regex = array();
		$vms_for_regex_list = array();
		$vms_instance_list = array();
		
		$regex_array =  explode("::", $regex_list);
		foreach ($regex_array as $regex_element) {
			$reg_array =  explode(";;", $regex_element);
			$tok_count = count($reg_array);
			if ($tok_count <> 3) {
				fprintf($log, "Invalid Regular Expression\n");
				break;
			}
			$container = $reg_array[0];
			$operator  = $reg_array[1];
			$pattern   = $reg_array[2];
			fprintf($log, "[c=$container, o=$operator, p=$pattern]\n");
			switch ($container) {
				case DATACENTER:
					$vms_for_regex = get_vms_for_container(DATACENTER_TYPE, "get_all_datacenters",
					                  $operator, $pattern, $address, $login, $password); 
				break;
				case ESX_CLUSTER:
					$vms_for_regex = get_vms_for_container(ESX_CLUSTER_TYPE, "get_all_esx_clusters", 
					                  $operator, $pattern, $address, $login, $password); 
				break;
				break;
				case ESX_SERVER:
					$vms_for_regex = get_vms_for_container(ESX_SERVER_TYPE, "get_all_esx_servers", 
					                  $operator, $pattern, $address, $login, $password); 
				break;
				case DATASTORE:
					$vms_for_regex = get_vms_for_container(DATASTORE_TYPE, "get_all_datastores",  
					                  $operator, $pattern, $address, $login, $password); 
				break;
				case RESOURCE_POOL:
					$vms_for_regex = get_vms_for_container(RESOURCE_POOL_TYPE, "get_all_resource_pool", 
					                  $operator, $pattern, $address, $login, $password); 
				break;
				case VAPP:
					$vms_for_regex = get_vms_for_container(VAPP_TYPE, "get_all_vapps", 
					                  $operator, $pattern, $address, $login, $password); 
				break;
				case FOLDER:
					$vms_for_regex = get_vms_for_container(FOLDER_TYPE, "get_all_folders", 
					                  $operator, $pattern, $address, $login, $password); 
				break;
				case VM_NAME:
					$vms_for_regex = get_vms_for_vmname($operator, $pattern, $address,
					                  $login, $password); 
				break;
				default :
					// bad entry in the database
					fprintf($log, "ERROR: Unsupported Regular Expression: $container\n");
				break;
			}
			if (empty($vms_for_regex_list)) {
				$vms_for_regex_list = $vms_for_regex;
			} else if (!empty($vms_for_regex)) {
				$vms_for_regex_list = array_intersect($vms_for_regex_list, $vms_for_regex);
			} else {
				//$vms_for_regex_list = $vms_for_regex_list;
			}
		}

		$vms_instance_list =  get_vms_instance_id($dbconn, $vms_for_regex_list);
		return $vms_instance_list;
	}
	
	#---------------------------------------------------------------------------------------------------
	# main program starts here
	#---------------------------------------------------------------------------------------------------
	$dbconn = pg_connect("port=5432 dbname=bpdb user=postgres connect_timeout=30");
	bp_bypass_cookie(3);
	setup_logging(); 
	
	fprintf($log, "START:\n______________________________________________________________________\n");
	
	/* Find list of schedules with vm_regex configured and associated ESX uuid. */
	$schedules_query  = "SELECT schedule_id, esx.esx_uuid, s.vm_regex";
	$schedules_query .= " FROM bp.schedules s";
	$schedules_query .= " JOIN (SELECT esx_uuid, name FROM bp.vmware_esx_servers";
	$schedules_query .= " LEFT JOIN bp.vmware_vcenter_assoc USING (esx_uuid)";
	$schedules_query .= "  WHERE vcenter_uuid IS NULL";
	$schedules_query .= "  UNION SELECT vcenter_uuid AS esx_uuid, name";
	$schedules_query .= "  FROM bp.vmware_vcenter_servers)";
	$schedules_query .= "  AS esx ON esx.esx_uuid = s.association_id";
	$schedules_query .= '  WHERE s.add_new_instances=true AND s.vm_regex <> \'\'';

	$schedules_res = pg_query($dbconn, $schedules_query);
	if ($schedules_res == false) {
		fprintf($log, "Failed to run query on the schedules table\n");
		bp_destroy_cookie();
		exit(1);
	}
        $schedule_query_nrows = pg_num_rows($schedules_res);

	fprintf($log, "No. of schedules with vm_regex: $schedule_query_nrows\n");
	for ($i = 0; $i < $schedule_query_nrows; $i++) {
		fprintf($log, "______________________________________________________________________\n");
		$schedule_query_row = pg_fetch_assoc($schedules_res, $i);
		$schedule_id = $schedule_query_row["schedule_id"];
		$host_uuid = $schedule_query_row["esx_uuid"];
		$vm_regex = $schedule_query_row["vm_regex"];
		fprintf($log, "schedule_id=$schedule_id, esx_uuid=$host_uuid, vm_regx=$vm_regex\n");
		/* Refresh the bp.vmware_connections table*/
		$res = bp_get_vm_info_no_regex($host_uuid, NULL, 0, 0);
		if ($res == false) {
			$err_msg = bp_error();
			fprintf($log, "$err_msg\n");
			continue;
		}
		/* Get login credentials for the host associated with the schedule */	
		$cred_query  = "SELECT login, password, address";
		$cred_query .= " FROM bp.vmware_connections";
		$cred_query .= " WHERE coalesce(vcenter_uuid,esx_uuid) = '$host_uuid'";
		$cred_res = pg_query($dbconn, $cred_query);
		if ($cred_res == false) {
			fprintf($log, "Failed to find login credentials for $host_uuid\n");
			continue;
		}
		if (pg_num_rows($cred_res) == 0) {
			fprintf($log, "No entry for vcenter_uuid: $host_uuid\n");
			continue;
		} 
		$cred_query_row = pg_fetch_assoc($cred_res, 0);
		$username = $cred_query_row["login"];
		$hash_password = $cred_query_row["password"];
		$address = $cred_query_row["address"];
		
		$vm_instance_list_db = array();
		$vm_instance_list_regex = array();
		$add_vms_to_schedule = array();	
		$add_comma_list = array();	
		$remove_comma_list = array();	
		$remove_vms_from_schedule = array();
			
		/* Get new list of VMs as per the configured list of regEx for this schedule */
		$vm_instance_list_regex = get_vms_for_regex_list($dbconn, $vm_regex, $address, 
		                                                 $username, $hash_password);
		
		/* Get existing list of VMs as per the db */ 
		$vm_instance_list_db = get_vms_for_schedule($dbconn, $schedule_id);
		
		/* Make list of instances to be removed from the schedule: db - regEx */
		if (empty($vm_instance_list_db)) {
			$remove_vms_from_schedule = array();
		} else if (empty($vm_instance_list_regex)) {
			$remove_vms_from_schedule = $vm_instance_list_db;
		} else {
			$remove_vms_from_schedule = array_diff($vm_instance_list_db, 
			                                       $vm_instance_list_regex);
		}

		/* Make list of instances to be added to the schedule: regEx - db */
		if (empty($vm_instance_list_regex)) {
			$add_vms_to_schedule = array();
		} else if (empty($vm_instance_list_db)) {
			$add_vms_to_schedule = $vm_instance_list_regex;
		} else {
			$add_vms_to_schedule = array_diff($vm_instance_list_regex,
			                                     $vm_instance_list_db);
		}

		// Remove instances from the schedules 
		if (!empty($remove_vms_from_schedule)) {
			$remove_comma_list = implode(",", $remove_vms_from_schedule);
			fprintf($log, "VMs to be Removed: $remove_comma_list\n");	
			delete_vms_from_schedule($dbconn, $remove_vms_from_schedule);
		} else {
			fprintf($log, "VMs to be Removed: [None]\n");
		}
		
		// Add instances to the schedules 
		if (!empty($add_vms_to_schedule)) {
			$add_comma_list = implode(",", $add_vms_to_schedule); 
			fprintf($log, "VMs to be Added: $add_comma_list.\n");	
			
			// First delete from other schedules, if any 
			delete_vms_from_schedule($dbconn, $add_vms_to_schedule);	
			fprintf($log, "Deleted before add.\n");
			$integer_instance_id_list = array();   
			foreach($add_vms_to_schedule as $value)
				$integer_instance_id_list[] = (int) $value; 
			// Add instances to this schedule 
			$res = bp_add_instances_to_vm_schedule($schedule_id, $integer_instance_id_list);
			if ($res == false) {
				$err_msg = bp_error();
				fprintf($log, "bp_add_instance_to_vm_schedule($schedule_id): $err_msg\n");
			} else {
				fprintf($log, "Added Successful\n");   
			} 
		} else {
			fprintf($log, "VMs to be Added: [None]\n");
		}
		
	}
	$msg = "### $program_name ends on ".date('Y-m-d H:i:s');
	fprintf($log, ":END\n");
	fprintf($log, "%s\n", $msg);
	bp_destroy_cookie();
	pg_close($dbconn);
	exit(0);
?>
