#!/bin/sh
#   dedup_estimate.sh
#
# 05/18/2017 acress - error if over limit of 30 min, add rpm/ui option
BPDIR=/usr/bp
eval `grep BPDIR /etc/default/bp.ini`
LOG=/var/log/unitrends-dedupchk.log
run_option="s"
# mydir=`pwd` # for testing
mydir=/var/local
rpm=0  # =0 from ui, =1 from rpm

dedup_conv_estimate() {
		lim=30
        # Estimate run-time
       meta_dir=`$BPDIR/bin/bputil -g "fileDedup" MetadataDirectory "" $BPDIR/bpinit/master.ini`
        metadata_size=`ls -l --block-size=M ${meta_dir}/metadataBlocks-0000000000000000|awk '{print $5}'|awk -F"M" '{print $1}'`
        is_virtual=`grep -c virtual_machine /var/opt/unitrends/platform_capabilities`
        if (( $is_virtual == 0 )); then
                (( estimated_runtime=metadata_size/105 ))
        else
                (( estimated_runtime=metadata_size/50 ))
        fi
        (( estimated_runtime=estimated_runtime/60 ))

        estimated_runtime=`echo "$estimated_runtime" | awk '{printf("%d\n",$1 + 0.5)}'`

        dbpart=`readlink -f /usr/bp/data | sed -e 's/.UnitrendsDataBase//'`
        mount |grep $dbpart | grep -q -w xfs
        if [ $? -ne 0 ]; then
			# it would take too long, increase the estimate
        	mount |grep $dbpart >>$LOG
			(( estimated_runtime=estimated_runtime+lim ))
        fi

        if [ "$run_option" == "s" ]; then
                if (( $estimated_runtime > $lim )); then
                        printf "WARNING: Migration exceeds $lim minutes window for this system\n" >> $LOG
						return 2
                fi
        fi
        printf "Raw estimated migration runtime is $estimated_runtime.\n" >>$LOG
        if (( $estimated_runtime <= 30 )); then
                estimated_runtime=30
        fi
        printf "The estimated migration runtime for this unit is $estimated_runtime minutes. Please keep in mind this is just an estimate and the migration could take longer.\n" >>$LOG
		return 0
}

upload_log() {
	logf=$1
    full_model=$(cat /etc/unitrends-asset)
    log_date=$(date +%m%d%y_%H%M)
    log_name=${full_model}_${log_date}
    ftp_loc="ftp://ftp.unitrends.com/incoming/dedupconvLogs"

    curl -s -T ${logf} ${ftp_loc}/${log_name} --user anonymous:
}

ret=0
echo "`cat /etc/unitrends-asset` `date`: $0 $1" >$LOG
if [ "$1" = "rpm" ]; then
   rpm=1
fi
rpm -q unitrends-rr >>$LOG
uname -a  >>$LOG
bputil=$BPDIR/bin/bputil
val=`$bputil -g fileDedup MetadataGeneration 0 /usr/bp/bpinit/master.ini`
if [ $val -ne 1 ]; then
   echo "Not using dedupNT" >>$LOG
   exit $ret
fi
val=`$bputil -g fileDedup StayDedupNT 0 /usr/bp/bpinit/master.ini`
if [ $val -eq 1 ]; then
   echo "Stay with dedupNT" >>$LOG
   exit $ret
fi
metautil=$BPDIR/bin/metadataUtil
$metautil --help 2>&1 |grep -q list-shadow
if [ $? -ne 0 ]; then
   echo "Have older $metautil missing list-shadow" >>$LOG
   if [ $rpm -eq 1 ]; then
     ret=1
   else
     metautil=$mydir/metadataUtil
     mkdir -p $mydir
     url=ftp://ftp.unitrends.com/support/acress/metadataUtil
     wget -q -O $metautil $url
     if [ $? -ne 0 ] || [ ! -f $metautil ]; then
	    echo "WARNING: Could not get $url" >>$LOG
	    metautil=$BPDIR/bin/metadataUtil
        ret=1
     else
        chmod 755 $metautil
     fi
   fi
fi
if [ $rpm -eq 1 ]; then
   MetadataDirectory=$($bputil -g fileDedup "MetadataDirectory" "" $BPDIR/bpinit/master.ini)
   lockBits=1
   lfile=$MetadataDirectory/metadataLock
   if [ "$MetadataDirectory" != "" ] && [ -f $lfile ]; then
     lockBits=$(stat --format=%s $lfile)
   fi
   if (( (lockBits & 0x1) != 0 )); then
     $metautil --unlock
   fi
   $metautil --list-shadow >>$LOG 2>&1
   rv=$?
   if (( (lockBits & 0x1) != 0 )); then
     $metautil --lock
   fi
   # $metautil --check --quick >>$LOG 2>&1
else
   $metautil --list-shadow >>$LOG 2>&1
   rv=$?
fi
if [ $rv -eq 0 ]; then
   echo "metadata health is ok" >>$LOG
else
   echo "WARNING: dedup conversion is needed, but metadata health check returned error $rv" >>$LOG
   if [ $rpm -eq 0 ]; then
      tail -n1 $LOG 1>&2
   fi
   ret=1
fi

if [ $rv -eq 0 ]; then
  dedup_conv_estimate
  rv=$?
  if [ $rv -eq 0 ]; then
    if [ $rpm -eq 0 ]; then
		printf "To optimize system performance, this appliance needs a software maintenance task that is run on the appliance during the update. \nThe estimated runtime for this task is 30 minutes.  If it takes longer than 30 minutes the task will be ended. \nIs it OK to proceed with the update now?\n"
	fi
	ret=5
  else
	echo "WARNING: dedup conversion is needed, but estimated dedup conversion runtime is too long ($estimated_runtime minutes)." >>$LOG
    if [ $rpm -eq 0 ]; then
		tail -n1 $LOG 1>&2
	fi
	ret=1
  fi
fi
if [ $rpm -eq 0 ]; then
   upload_log $LOG
fi
exit $ret
