#!/bin/bash

#####################################################
# This script will extract the metadata stored in  
# dedupNT and migrate it to DB
# The script options are as follows:
#   -s for silient mode,no prompts, will stop if data in tables
#   -m send mail
#   -f force truncation of the tables, allows silent mode to continue
#   -i ignore errors when loading the backups_hash_values errors
#   -b skip loading the hash_values records and start loading backups_hash_values
#   -c cleanup disable_rrc and motd files from failed runs
# A sample call would be:
#   migrate_nt2db.sh will prompt user, no email
#   migrate_nt2db.sh -m will prompt user, will attempt to email on error or completion
#   migrate_nt2db.sh -s will not prompt user, will exit if data in tables
#   migrate_nt2db.sh -i will not stop if errors found while loading backups_hash_values, a WARNING will be written to the log
#   migrate_nt2db.sh -b will start the load of the bakcups_hash_vallues data anddd skip loading hash_values
#   migrate_nt2db.sh -sm will not prompt user, will attemp to email on error or commpletion, will exit if data in tables
#   migrate_nt2db.sh -sf will not prompt user, will truncate data in tables
#   migrate_nt2db.sh -smfi will not prompt user,will attempt to email on error or completion, will truncate data in tables, will ignore backups errors
# Return codes for this scripts are as follows:
#   1 - system not running dedupNT
#   2 - unable to access postgres
#   3 - error running psql command
#   4 - error found in log file
#   5 - invalid option
#   6 - psql command not found
#   7 - user stopped execution
#   8 - lock file found
#   9 - not enought space in db partition
#   10 - error in subshell
#   11 - data found in database tables
#   12 - no data found in hash_values table trying to load backups_hash_values records only
#   13 - not in a screen session
#   14 - failed migration
#   15 - found duplicate hashes
#   16 - backup count mismatch
#####################################################
# 05/19/2017 acress - added -r rpm option 
# 05/22/2017 acress - added unlock before main metadataUtil function

# Special bash variable to count how long script has run
SECONDS=0
script_name=${0##*/}

# We've seen psql timeout errors when the system is very busy
# we can increase the default time of 5 seconds to 300 seconds
PGCONNECT_TIMEOUT=300
export PGCONNECT_TIMEOUT

email_address="support@unitrends.com"
BPDIR=`/usr/bp/bin/bputil -g "Location Information" "BPDIR" "" /etc/default/bp.ini`
DATE=`date +%F-%H:%M:%S`
database_dir=$BPDIR/data
meta_dir=`$BPDIR/bin/bputil -g "fileDedup" MetadataDirectory "" $BPDIR/bpinit/master.ini`
meta_dir=`echo ${meta_dir%/}`
avail_db_dir=$(df -Pk $database_dir/ | tail -1 | awk '{print $4}')
present_dir=`pwd`
out_dir="/var/log"
LOG=$out_dir/${script_name}.log
metautil=$BPDIR/bin/metadataUtil
metalockBits=1
maintenance_work_mem=200000
processor_count=`grep -c processor /proc/cpuinfo`
(( max_workers_process_count=processor_count*3 ))
mem_kb=`grep MemTotal /proc/meminfo | grep -E -o "[0-9]+"`
let mem_mb=$mem_kb/1024
start_time=0
estimated_runtime=0
metadata_size=0
elapsed_time=0
work_mem=200000
rc=0
workers_process_count=0
data_found=0
ignore_prompt=0
backups_only_prompt=0
force_trunc_prompt=0
cleanup_failed_prompt=0
no_records_backup=0
run_option=""
rpm=0
send_mail=0
run_verbiage="NOP"
migration_folder="/backups/migration_nt_db"
worker_error_file=${migration_folder}/worker_error_found
snmp_status=${migration_folder}/started
backup_list_file=${migration_folder}/${script_name}_backup_list.out
backup_tbl_list_file=${migration_folder}/${script_name}_backup_tbl_list.out
check_metadata=`$BPDIR/bin/bputil -g "fileDedup" MetadataGeneration "" $BPDIR/bpinit/master.ini`
check_screen=""
dedup_cap_found=`grep -wc 'dedupNT' /var/opt/unitrends/platform_capabilities|tr -d ' \t\n\r\f'`
database_name="bpdb"
database_user="postgres"
dpu_asset=`/usr/local/bin/dpu asset`
dpu_version=`/usr/local/bin/dpu version`
bp_version=`/usr/local/bin/dpu version | grep Version | tr -d "Version " |awk -F"." '{print $1$2$3}'|awk -F"-" '{print $1}'`
ip_address=`ifconfig eth0 | grep 'inet addr:' | cut -d: -f2 | awk '{ print $1}'`
total_num_backups=0
total_num_backups_table=0
current_num_backups=0
failed_migration=0
duplicate_hashes="f"

USAGE="Usage: $script_name \n
 The script options are as follows: \n
   -s for silient mode,no prompts, will stop if data in tables \n
   -m send mail \n
   -f force truncation of the tables, allows silent mode to continue \n
   -c cleanup disable_rrc and motd files from previously failed runs \n
 A sample call would be: \n
   migrate_nt2db.sh will prompt user, no email \n
   migrate_nt2db.sh -m will prompt user, will attempt to email on error or completion \n
   migrate_nt2db.sh -s will not prompt user, will exit if data in tables \n
   migrate_nt2db.sh -sm will not prompt user, will attemp to email on error or commpletion, will exit if data in tables \n
   migrate_nt2db.sh -sf will not prompt user, will truncate data in tables \n
   migrate_nt2db.sh -smf will not prompt user,will attempt to email on error or completion, will truncate data in tables \n"
lockFile=/var/lock/"${script_name}"-lock

# make sure the log file is present
touch $LOG

clean_up (){
	echo -ne "\n"|tee -a $LOG
	###################
	# turn on autovac
	###################
	update_autovac on
	printf "Resetting the /etc/motd file and removing the $BPDIR/lists.dir/.disable_rrc.\n"|tee -a $LOG
	if [ -e /etc/motd_pre_migration ]; then
		yes|cp -rf /etc/motd_pre_migration /etc/motd >> $LOG 2>&1
		mv /etc/motd_pre_migration /etc/motd_pre_migration_done >> $LOG 2>&1
	fi
	rm -f $BPDIR/lists.dir/.disable_rrc >> $LOG 2>&1
	if (( $failed_migration == 0 )) && [ ! -e $worker_error_file ]; then
		mv ${snmp_status} ${new_snmp_status} > /dev/null 2>&1
		rm -f $lockFile ${migration_folder}/started $worker_error_file ${migration_folder}/*failed $BPDIR/lists.dir/.disable_rrc >/dev/null 2>&1
	else
		rm -f $lockFile ${migration_folder}/started $BPDIR/lists.dir/.disable_rrc >/dev/null 2>&1
	fi
	duration=$SECONDS
	duration=`echo $(($duration / 60))`
	printf "Migration script ran in ${duration} minutes...\n"|tee -a $LOG
	printf "END: `date +%F-%H:%M:%S` \n" >> $LOG
	email_status
	rm -f $lockFile ${migration_folder}/started  >/dev/null 2>&1
}

clean_up_trap (){
	echo -ne "\n"|tee -a $LOG
	touch $worker_error_file
	printf "The script was aborted...\n"|tee -a $LOG
	###################
	# turn on autovac
	###################
	update_autovac on
	printf "Resetting the /etc/motd file and removing the $BPDIR/lists.dir/.disable_rrc.\n"|tee -a $LOG
	if [ -e /etc/motd_pre_migration ]; then
		yes|cp -rf /etc/motd_pre_migration /etc/motd >> $LOG 2>&1
		mv /etc/motd_pre_migration /etc/motd_pre_migration_done >> $LOG 2>&1
	fi
	rm -f $BPDIR/lists.dir/.disable_rrc >> $LOG 2>&1
	if (( $failed_migration == 0 )) && [ ! -e $worker_error_file ]; then
		mv ${snmp_status} ${new_snmp_status} > /dev/null 2>&1
		rm -f $lockFile ${migration_folder}/started $worker_error_file ${migration_folder}/*failed $BPDIR/lists.dir/.disable_rrc >/dev/null 2>&1
	else
		rm -f $lockFile ${migration_folder}/started $BPDIR/lists.dir/.disable_rrc >/dev/null 2>&1
	fi
	duration=$SECONDS
	duration=`echo $(($duration / 60))`
	printf "Migration ran in ${duration} minutes...\n"|tee -a $LOG
	printf "END: `date +%F-%H:%M:%S` \n" >> $LOG
	email_status
	rm -f $lockFile ${migration_folder}/started  >/dev/null 2>&1
}

trap clean_up_trap SIGINT SIGHUP SIGTERM SIGALRM SIGQUIT

update_autovac () {
        param_name="autovacuum"
        param_value="$1"
        case "$param_value" in
                on) old_param="off";;
                off) old_param="on";;
                *)  printf "Invalid parameter passed to update autovac.\nWill exit now...\nRC=8\n"|tee -a $LOG; 
		    email_status; clean_up;exit 8 ;;
        esac
        printf "Turning autovacuum $1...\n"|tee -a $LOG
        sed -i -r -e "s/^\s*${param_name}\s*=\s*${old_param}/$param_name = $param_value /" $database_dir/postgresql.conf
	sudo -iu postgres pg_ctl reload -D $database_dir >> $LOG 2>&1
}

update_wal_buffer () {
        param_name="wal_buffers"
        param_value="500MB"
        printf "Tunning wal_buffer to $param_value...\n"|tee -a $LOG
        sed -i -r -e "s/^\s*${param_name}\s*=\s*/$param_name = $param_value #/" $database_dir/postgresql.conf
	sudo -iu postgres pg_ctl restart -m fast -D $database_dir >> $LOG 2>&1
}

check_storage () {
	if (( $avail_db_dir < $used_metadata_dir )); then
		failed_migration=1
		printf "Not enough space in the database partition $database_dir ...\n"|tee -a $LOG
		printf "$database_dir space available is $avail_db_dir space needed is $used_metadata_dir...\n"|tee -a $LOG
		printf "RC=9\n" >> $LOG
		# Update progress file for snmp reporting
		new_snmp_status=${migration_folder}/storage_failed
		mv ${snmp_status} ${new_snmp_status}
		email_status
		clean_up
		exit 9
	fi
}
proceed_check (){
	printf "BEGIN: `date +%F-%H:%M:%S` \n" >> $LOG

	# call check_is_up function to make sure postgres is running
	if [[ `check_db_is_up` -ne 0 ]]; then
		printf "Postgres seems to be down...\n"| tee -a $LOG
		printf "Starting the postgres database now...\n"|tee -a $LOG
		sudo -iu postgres pg_ctl start -D $database_dir >> $LOG 2>&1
		sleep 5
	else
			check_bpdb_is_available
	fi
	if [[ `check_db_is_up` -ne 0 ]]; then
		failed_migration=1
		printf "Postgres seems to be down...\n"| tee -a $LOG
		printf "Please look at the $LOG log file for more details...\n"|tee -a $LOG
		printf "RC=2\n"|tee -a $LOG
		DATE=`date +%F-%H:%M:%S`
		printf "$DATE \n" >> $LOG
		# Update progress file for snmp reporting
		new_snmp_status=${migration_folder}/psql_failed
		clean_up
		exit 2
	fi

	if [[ "${check_metadata}" -eq "" ]] || [ "${check_metadata}" -eq "0" ]; then
		failed_migration=1
		printf "MetadataGeneration is not set in the master.ini file.\nThis system is not running dedupNT exiting now...\n"|tee -a $LOG
		printf "RC=1\n"|tee -a $LOG
		DATE=`date +%F-%H:%M:%S`
		printf "$DATE \n" >> $LOG
		# Update progress file for snmp reporting
		new_snmp_status=${migration_folder}/conf_failed
		clean_up
		exit 1
	fi
	if (( $dedup_cap_found == 0 )); then
		failed_migration=1
		printf "dedupNT is not set in the capability file.\nThis system is not running dedupNT exiting now...\n"|tee -a $LOG
		printf "RC=1\n"|tee -a $LOG
		DATE=`date +%F-%H:%M:%S`
		printf "$DATE \n" >> $LOG
		# Update progress file for snmp reporting
		new_snmp_status=${migration_folder}/conf_failed
		clean_up
		exit 1
	fi
	if [[ ! -d $meta_dir ]]; then
		failed_migration=1
		printf "dedupNT $meta_dir directory is not accessible...\n"|tee -a $LOG
		printf "RC=1\n"|tee -a $LOG
		DATE=`date +%F-%H:%M:%S`
		printf "$DATE \n" >> $LOG
		# Update progress file for snmp reporting
		new_snmp_status=${migration_folder}/conf_failed
		clean_up
		exit 1
	fi

	used_metadata_dir=$(du -Pk $meta_dir/ | awk '{print $1}')
	# make sure there's enough space in the db partition to hold new data
	check_storage

	# Estimate run-time
	metadata_size=`ls -l --block-size=M ${meta_dir}/metadataBlocks-0000000000000000|awk '{print $5}'|awk -F"M" '{print $1}'`
	is_virtual=`grep -c virtual_machine /var/opt/unitrends/platform_capabilities`
	if (( $is_virtual == 0 )); then
		(( estimated_runtime=metadata_size/105 ))
	else
		(( estimated_runtime=metadata_size/50 ))
	fi
	(( estimated_runtime=estimated_runtime/60 ))
	
	estimated_runtime=`echo "$estimated_runtime" | awk '{printf("%d\n",$1 + 0.5)}'`
	if [ "$run_option" == "s" ]; then
		if (( $estimated_runtime > 120 )); then
			printf "WARNING: Migration exceeds 2 hours window for this system\n" >> $LOG
		fi
	fi
	printf "Raw estimated migration runtime is $estimated_runtime.\n"|tee -a $LOG
	if (( $estimated_runtime <= 15 )); then
		estimated_runtime=15
	fi
	printf "The estimated migration runtime for this unit is $estimated_runtime minutes. Please keep in mind this is just an estimate and the migration could take longer.\n"|tee -a $LOG

        printf "This script will stop all processes now...\n"|tee -a $LOG
	printf "Once the script starts there is no way to migrate the data back to dedupNT.\n"|tee -a $LOG
	if [ "$run_option" != "s" ]; then
		printf "Would you like to proceed?[Yy]...\n"|tee -a $LOG
		read answer
		printf "$answer \n" >> $LOG
		if [[ $answer != [Yy] ]]; then
			printf "Exiting script now...\n"|tee -a $LOG
			printf "RC=7\n"|tee -a $LOG
			# Update progress file for snmp reporting
			new_snmp_status=${migration_folder}/usr_exit
			mv ${snmp_status} ${new_snmp_status} >> $LOG 2>&1
			clean_up
			exit 7
		fi
	fi

	# Set the disable_rrc file to show in the UI maintenance is ongoing
	if [ $rpm -eq 1 ]; then
		touch $BPDIR/lists.dir/.disable_rrc
	else
	 if [ -f $BPDIR/lists.dir/.disable_rrc ]; then
		printf "Found $BPDIR/lists.dir/.disable_rrc ...\nPlease verify the system is stable and try again...\n"|tee -a $LOG
		printf "RC=1\n"|tee -a $LOG
		DATE=`date +%F-%H:%M:%S`
		printf "$DATE \n" >> $LOG
		new_snmp_status=${migration_folder}/conf_failed
		clean_up
		exit 1
	 else
		touch $BPDIR/lists.dir/.disable_rrc
	 fi
	fi
	printf "A maintenance operation was started by Unitrends Support\nDo not restart or power-off without contacting Unitrends Support" >> $BPDIR/lists.dir/.disable_rrc
	printf "Process Started: `date +%F-%H:%M:%S` \n" >> $BPDIR/lists.dir/.disable_rrc

	# Setting the /etc/motd file message also
	cp -an /etc/motd /etc/motd_pre_migration|tee -a $LOG
	printf "Setting the message of the day now...\n"|tee -a $LOG
	printf "###########################################\n">/etc/motd
	printf "###########################################\n">>/etc/motd
	printf "#This system is undergoing a dedupNT to   #\n">>/etc/motd
	printf "#dedupDB migration.  Please refrain from  #\n">>/etc/motd
	printf "#stopping or starting any processes.      #\n">>/etc/motd
	printf "#Contact Unitrends Support with questions.#\n">>/etc/motd
	printf "###########################################\n">>/etc/motd
	printf "###########################################\n">>/etc/motd
}


email_status (){
	if (( $send_mail == 1 )); then
		# spawn a subshell to pass the necessary command to telent to the mail server an email
		(
			msg1=`head -20 $LOG`
			msg2=`tail -150 $LOG`
			new_snmp_status=`echo ${new_snmp_status##*/}`
			echo "open unitrends.mail.eo.outlook.com 25"
			sleep 8
			echo "helo"
			echo "mail from: root@`hostname`"
			sleep 2
			echo "rcpt to: $email_address"
			sleep 2
			echo "data"
			sleep 2
			echo "subject: ${dpu_asset}-${new_snmp_status}"
			echo 
			echo
			printf "${ip_address}\n${dpu_asset}\n${dpu_version}\n${msg1}\n${msg2}\n"
			sleep 5
			echo "."
			sleep 5
			echo "quit") | telnet >>$LOG 2>&1
	fi
}

#
#Try to find where psql is. If not found then exit
#
check_psql (){
/usr/bin/which psql > /dev/null 2>&1
rc=$?
if [[ $rc -eq 0 ]]; then
        psql_path="`/usr/bin/which psql`"
        psql_path=`echo ${psql_path%psql}`
        PSQL=`echo $psql_path/psql -v ON_ERROR_STOP=1`
else
	failed_migration=1
        printf "Could not find psql command in the path, exiting now.\n" >> $LOG
	# Update progress file for snmp reporting
	new_snmp_status=${migration_folder}/psql_failed
	clean_up
        exit 6
fi
}

check_rc (){
	if (( $rc != 0 )); then
		failed_migration=1
		touch $worker_error_file
		echo -ne "\n"|tee -a $LOG
		printf "Command returned an error...\nPlease check the $LOG for more info...\n"|tee -a $LOG
		printf "ERROR: !!!!!!!!Migration failed!!!!!!!!"
		printf "Exiting now...\n"|tee -a $LOG
		printf "RC=4\n" >> $LOG
		# Update progress file for snmp reporting
		new_snmp_status=${migration_folder}/failed
		clean_up
		exit 4
	fi
	check_worker_error
}
check_worker_rc (){
	if (( $rc != 0 )); then
		failed_migration=1
		echo -ne "\n"|tee -a $LOG
		touch $worker_error_file
		printf "Command returned an error for backup ${backupno} ...\nPlease check the $LOG for more info...\n"|tee -a $LOG
		printf "ERROR: !!!!!!!!Migration failed!!!!!!!!"
		printf "Exiting now...\n"|tee -a $LOG
		printf "RC=10\n" >> $LOG
		# Update progress file for snmp reporting
		new_snmp_status=${migration_folder}/worker_failed
		clean_up
		exit 10
	fi
}

check_worker_rc_backups (){
	if (( $rc != 0 )); then
		echo -ne "\n"|tee -a $LOG
		printf "WARNING: Command returned an error for backup ${backupno} ...\nPlease check the $LOG for more info...\n"|tee -a $LOG
		# Update progress file for snmp reporting
	fi
}

check_worker_error (){
	if [[ -e $worker_error_file ]]; then
		failed_migration=1
		echo -ne "\n"|tee -a $LOG
		printf "ERROR: A subshell had an error... Exiting now... \n"|tee -a $LOG
		printf "ERROR: !!!!!!!!Migration failed!!!!!!!!"
		printf "RC=10 \n" >> $LOG
		# Update progress file for snmp reporting
		new_snmp_status=${migration_folder}/worker_failed
		clean_up
		exit 10
	fi
}

stop_services (){
        printf "Stopping all processes...\n" |tee -a $LOG
        /etc/init.d/bp_rcscript stop >> $LOG 2>&1

        printf "Running /usr/bp/bin/stop_db.sh...\n"|tee -a $LOG
        /usr/bp/bin/stop_db.sh >> $LOG 2>&1

        printf "Stopping crond processes...\n" |tee -a $LOG
        service crond stop >> $LOG 2>&1
}

start_services (){
	opt=$1
	if [ "$run_option" != "s" ]; then
		printf "Would you like to start all services now?[Yy]...\n"|tee -a $LOG
		read answer
		printf "$answer \n" >> $LOG
		if [[ $answer != [Yy] ]]; then
			printf "Exiting script now...\nPlease remember to start all bp processes and cron...\n"|tee -a $LOG
			printf "RC=7\n" >> $LOG
			clean_up
			exit 7
		fi
	fi
	stop_services
        printf "Starting httpd processes...\n" |tee -a $LOG
        service httpd start >> $LOG 2>&1

        printf "Starting crond processes...\n" |tee -a $LOG
        service crond start >> $LOG 2>&1

	if [ "$opt" = "rpm" ]; then
        printf "Starting database...\n" |tee -a $LOG
        /usr/bp/bin/start_db.sh >> $LOG 2>&1
	else
        printf "Starting all processes...\n" |tee -a $LOG
        /etc/init.d/bp_rcscript start >> $LOG 2>&1
	fi
	clean_up
}

# Function to check if postgres is up
check_db_is_up (){
        rc=0
        sudo -iu postgres pg_ctl status -D $database_dir >> $LOG 2>&1
        rc=$?
        echo $rc
}

check_bpdb_is_available (){
        wait_time=0
        ready_found=0
        recovery_found=0
        recovery_msg=0
        total_wait_time=180
        postgres_log_ln=0
        current_postgres_log_ln=0
        postgres_log=`ls -t $database_dir/pg_log|awk '{print $1}'|head -1`
        # Wait no more than 5 minutes or 300 seconds for the datbase to be ready
        printf "Running select statement to check database availability...\n" |tee -a $LOG
        $PSQL -U $database_user $database_name -c"select 1 from bp.backups limit 1" > /dev/null 2>&1
        ready_found=$?
        if [[ $ready_found != 0 ]]; then
                printf "Database is not currently available.  Will retry in a few seconds...\n" | tee -a $LOG
                current_postgres_log_ln=`wc -l $database_dir/pg_log/$postgres_log|awk '{print $1}'`
                while (( $wait_time < $total_wait_time ))
                do
                        postgres_log_ln=`wc -l $database_dir/pg_log/$postgres_log|awk '{print $1}'`
                        (( postgres_log_ln_diff=current_postgres_log_ln-postgres_log_ln ))
                        # Look for certain keywords if found exit
                        tail_lines=`echo "-"$postgres_log_ln_diff`
                        printf "Running select statement to check database availability...\n" |tee -a $LOG
                        $PSQL -U $database_user $database_name -c"select 1 from bp.backups limit 1" > /dev/null 2>&1
                        ready_found=$?
                        # if we haven't found the "is ready" message keep waiting
                        if [[ $ready_found == 0 ]]; then
                                printf "Database is now available...\n" |tee -a $LOG
                                break;
                        fi
                        # Look for recovery mode and just display a message once
                        tail $tail_lines $database_dir/pg_log/$postgres_log | grep -q "recovery in progress"
                        recovery_found=$?
                        if [[ $recovery_found == 0 ]] && (( $recovery_msg == 0 )); then
                                echo "NOTICE: Database is starting up.  Please stand-by." | tee -a $LOG
                                recovery_msg=1
                                total_wait_time=300
                        fi
                        sleep 20
                        (( wait_time=wait_time+20 ))
                done
        fi
        # if no message is found, postgres could be in recovery pending for a long time and we need to stop
        printf "Running select statement to check database availability...\n" |tee -a $LOG
        $PSQL -U $database_user $database_name -c"select 1 from bp.backups limit 1" > /dev/null 2>&1
        ready_found=$?
        if [[ $ready_found != 0 ]]; then
		failed_migration=1
                rc=2
                echo "ERROR: Please verify the $database_dir/pg_log/$postgres_log log. Database seems to be down, aborting..." | tee -a $LOG
                printf "Please look at the $LOG log file for more details...\n"|tee -a $LOG
                printf "RC=$rc \n"|tee -a $LOG
		# Update progress file for snmp reporting
		new_snmp_status=${migration_folder}/psql_failed
		clean_up
                exit $ret
        fi
}

check_elapsed_time (){
	(( elapsed_time=SECONDS-start_time ))
	elapsed_time=`echo $(($elapsed_time / 60))`
	printf "$1 ran in ${elapsed_time} minutes...\n" >> $LOG
}

migrate_dedupNT (){

# use metautil to extract the list of backups that have metadata in dedupNT
start_time=$SECONDS
if (( (metalockBits & 0x1) != 0 )); then
	$metautil --unlock >> $LOG 2>&1
fi
printf "Getting backups in dedupNT metadata using $metautil -b :\n" | tee -a $LOG
$metautil -b > $backup_list_file 2>>$LOG
rc=$?
check_elapsed_time "extract_backups_list"
check_rc
total_num_backups=`wc -l $backup_list_file|awk '{print $1}'`
total_num_backups_table=`$PSQL -U $database_user $database_name -tq -c"select count(*) from bp.backups where properties & 1024 = 1024 or properties & 512 = 512;"|head -1|tr -d '[:space:]'`

printf "INFO: Total number of backups found with hash or dedup references: $total_num_backups_table\n" >> $LOG
printf "Total number of backups to migrate: $total_num_backups \n"|tee -a $LOG

# Used for informationl purposes
if (( $total_num_backups < $total_num_backups_table )); then
	printf "INFO: The number of migrated backups does not match.\n" >> $LOG
	printf "INFO: There were $total_num_backups_table out of which only $total_num_backups would be migrated\n" >> $LOG
	$PSQL -U $database_user $database_name -tq -c"select backup_no from bp.backups" > $backup_tbl_list_file
#	Comparison logic
#	diff -iEBU $(wc -l < $backup_tbl_list_file) $backup_tbl_list_file $backup_list_file | grep '^-' | sed 's/^-//g' >> $LOG
fi

printf "`date +%F-%H:%M:%S`: Total number of backups to optimize: $total_num_backups \n" >> $BPDIR/lists.dir/.disable_rrc

if [[ `$PSQL -U $database_user $database_name -tAc "select exists(select 1 from bp.hash_values)"` == "t" ]]; then 
	printf "There is already data in the bp.hash_values table...\n"|tee -a $LOG
	data_found=1
else
	if (( $backups_only_prompt == 1 )); then
		printf "You are trying to load the backups_hash_values data only but there are no records in the hash_values table...\n"|tee -a $LOG
		printf "Exiting now..\n"|tee -a $LOG
		DATE=`date +%F%H%M%S`
		printf "$DATE \n" >> $LOG
		# Update progress file for snmp reporting
		new_snmp_status=${migration_folder}/backups_only_failed
		clean_up
		exit 12
	fi
fi
if [[ `$PSQL -U $database_user $database_name -tAc "select exists(select 1 from bp.backups_hash_values)"` == "t" ]]; then 
	printf "There is already data in the bp.backups_hash_values table...\n"|tee -a $LOG
	data_found=1
fi

# Update progress file for snmp reporting
new_snmp_status=${migration_folder}/in_progress
mv ${snmp_status} ${new_snmp_status} >> $LOG 2>&1
snmp_status=${new_snmp_status}


if (( $data_found == 1 )); then
	if [ "$run_option" != "s" ]; then
		printf "Would you like to purge the existing data in the database?[Yy]\n"|tee -a $LOG
		read answer
		printf "$answer \n" >> $LOG
	else
		if (( $force_trunc_prompt == 0 )); then
			printf "Data found in database tables...\nExiting now\n"|tee -a $LOG
			printf "RC=11\n">>$LOG
			DATE=`date +%F%H%M%S`
			printf "$DATE \n" >> $LOG
			# Update progress file for snmp reporting
			new_snmp_status=${migration_folder}/data_found_failed
			clean_up
			exit 11
		fi
		answer="Y"
	fi
        if [[ $answer != [Yy] ]]; then
                printf "Exiting script now...\n"|tee -a $LOG
                printf "RC=7\n" >> $LOG
		DATE=`date +%F%H%M%S`
		printf "$DATE \n" >> $LOG
		# Update progress file for snmp reporting
		new_snmp_status=${migration_folder}/usr_exit
		clean_up
                exit 7
	else 
		if (( $backups_only_prompt == 0 )); then
			printf "Truncating the bp.hash_values table...\n"|tee -a $LOG
			start_time=$SECONDS
			$PSQL -U $database_user $database_name <<-EOF >> $LOG 2>&1
			TRUNCATE TABLE bp.hash_values cascade;
			EOF
			rc=$?
			check_elapsed_time "trunc_hash"
			check_rc
		fi
		printf "Truncating the backups_hash_values table...\n"|tee -a $LOG
		start_time=$SECONDS
		$PSQL -U $database_user $database_name <<-EOF >> $LOG 2>&1
		TRUNCATE TABLE bp.backups_hash_values cascade;
		EOF
		rc=$?
		check_elapsed_time "trunc_backups_hash"
		check_rc
        fi
fi

stop_services

/usr/sbin/lsof $meta_dir/* >/dev/null
rc=$?
if (( $rc == 0 )); then
	printf "There seem to be other processes accessing the metadata repo.  Please cancel any processes doing so before the migration.\n"|tee -a $LOG
	/usr/sbin/lsof $meta_dir/* |tee -a $LOG 
	DATE=`date +%F%H%M%S`
	printf "$DATE \n" >> $LOG
	# Update progress file for snmp reporting
	new_snmp_status=${migration_folder}/conf_failed
	clean_up
	exit 1
fi

MetadataDirectory=$(${BPDIR}/bin/bputil -g fileDedup        \
    "MetadataDirectory" "" ${BPDIR}/bpinit/master.ini)
metalockBits=1
lfile=$MetadataDirectory/metadataLock
if [ "$MetadataDirectory" != "" ] && [ -f $lfile ]; then
	metalockBits=$(stat --format=%s $lfile)
fi

$metautil --help 2>&1 |grep -q list-shadow
if [ $? -eq 0 ]; then
	start_time=$SECONDS
	if (( (metalockBits & 0x1) != 0 )); then
	   printf "Running $metautil --unlock \n"|tee -a $LOG
	   $metautil --unlock >> $LOG 2>&1
	   rc=$?
	   check_elapsed_time "meta_unlock"
	   check_rc
	fi
	printf "Running a $metautil --list-shadow \n"|tee -a $LOG
	printf "`date +%F-%H:%M:%S`: Running a check on the hash data... \n" >> $BPDIR/lists.dir/.disable_rrc
	start_time=$SECONDS
	$metautil --list-shadow >>$LOG 2>&1
	rc=$?
	check_elapsed_time "meta_list_shadow"
	check_rc
else
	printf "Running a $metautil --check --quick --wait-on-lock \n"|tee -a $LOG
	printf "`date +%F-%H:%M:%S`: Running a check on the hash data... \n" >> $BPDIR/lists.dir/.disable_rrc
	start_time=$SECONDS
	$metautil --check --quick --wait-on-lock >> $LOG 2>&1
	rc=$?
	check_elapsed_time "meta_check"
	check_rc
fi

printf "Starting the postgres database...\n"|tee -a $LOG
sudo -iu postgres pg_ctl start -D $database_dir >> $LOG 2>&1
sleep 5
update_wal_buffer
check_bpdb_is_available

# call check_is_up function to make sure postgres is running
if [[ `check_db_is_up` -ne 0 ]]; then
	printf "Postgres seems to be down...\nExiting now...\n"| tee -a $LOG
	printf "Please look at the $LOG log file for more details...\n"|tee -a $LOG
	printf "RC=2\n"|tee -a $LOG
	DATE=`date +%F%H%M%S`
	printf "$DATE \n" >> $LOG
	# Update progress file for snmp reporting
	new_snmp_status=${migration_folder}/psql_failed
	clean_up
	exit 2
fi

###################
# turn off autovac
###################
printf "Turning autovac off for postgres...\n"|tee -a $LOG
update_autovac off

if (( $backups_only_prompt == 0 )); then

	# drop hash_values indexes to speed up load performance
	printf "Dropping indexes for the bp.hash_values table...\n"|tee -a $LOG
	start_time=$SECONDS
	$psql_path/psql -U $database_user $database_name <<-EOF >> $LOG 2>&1
	SET search_path to bp;
	DROP INDEX bp.hash_values_sis_offset_index;
	ALTER TABLE bp.hash_values DROP CONSTRAINT hash_values_hash_type_key;
	ALTER TABLE bp.hash_values DROP CONSTRAINT hash_type_length_value_unique;
	ALTER TABLE bp.hash_values DROP CONSTRAINT hash_value_length_type_unique;
	ALTER TABLE bp.hash_values DROP CONSTRAINT hash_values_pkey CASCADE;
	EOF
	rc=$?
	check_elapsed_time "drop_hash_index"
	check_rc

	# use metautil to extract hash_values information
	printf "Extracting and loading hash_value data using $metautil -L ...\n"|tee -a $LOG
	printf "`date +%F-%H:%M:%S`: Extracting and loading hash data... \n" >> $BPDIR/lists.dir/.disable_rrc
	start_time=$SECONDS
	# Need to delete informational line
	#sed -i -e '1d' -e '$d' $tmp_hash_file >> $LOG 2>&1

	# Format the lines to conform with COPY utility used to load hash_values
	# If inSIS equals "n", then sis_offset should be NULL and compressed_length should be 0.
	# If inSIS equals "y", then sis_offset should equal SISoffset and compressed_length should equal compressedLength.
	# 4-compressed_length 5-sisoffset 6-insis
	check_worker_error
	$metautil -L 2>>$LOG | sed -e '1d' -e '$d'|awk -F' ' '{if ($6 == "n") print $1 " " $2 " " $3 " \\\\x"$7 " " "\\N" " " "0" ; else print $1 " " $2 " " $3 " \\\\x"$7 " " $5 " " $4 ;}'|sed 's/0x//g' | \
	$PSQL -U $database_user $database_name -c"SET work_mem to $work_mem;copy bp.hash_values from STDIN (DELIMITER ' ');" 2>>$LOG
	test_piperc=${PIPESTATUS[*]}
	check_elapsed_time "hash_migration"
	for rc in ${test_piperc[@]}
	do
		check_rc
	done
	check_worker_error

	printf "Searching for duplicate hashes.\nIf any duplicates are found a metadataUtil full repair would need to be executed and the migration re-run.\n"|tee -a $LOG
	start_time=$SECONDS
	duplicate_hashes=`$PSQL -tq -U $database_user $database_name -c"select exists(select 1 from bp.hash_values group by hash_type, hashed_length, hash_value having count(*) > 1)"|head -1|tr -d '[:space:]' 2>>$LOG`
	check_elapsed_time "serch_dup_hashes"
	if [[ "${duplicate_hashes}" == "t" ]]; then
		printf "ERROR: Found duplicate hashes. Migration will stop now.\nPlease run a metadataUtil full repair before attempting to run the migration again.\n"|tee -a $LOG
		DATE=`date +%F%H%M%S`
		printf "$DATE \n" >> $LOG
		printf "RC=14\n" >> $LOG
		clean_up
		exit 14
	fi

	printf "`date +%F-%H:%M:%S`: Finished loading hash data... \n" >> $BPDIR/lists.dir/.disable_rrc

	printf "Recreating primary key for bp.hash_values...\n"|tee -a $LOG
	start_time=$SECONDS
	$PSQL -U $database_user $database_name <<-EOF >> $LOG 2>&1
	SET maintenance_work_mem to $maintenance_work_mem;
	SET search_path to bp;
	CHECKPOINT;
	ALTER TABLE bp.hash_values ADD CONSTRAINT hash_values_pkey PRIMARY KEY (hash_id);
	ALTER TABLE bp.backups_hash_values ADD CONSTRAINT backups_hash_values_hash_id_fkey FOREIGN KEY (hash_id) REFERENCES bp.hash_values(hash_id) ON UPDATE RESTRICT ON DELETE RESTRICT;
	ANALYZE VERBOSE bp.hash_values;
	EOF
	rc=$?
	check_elapsed_time "create_pk_hash"
	check_rc
else
	printf "Skipping loading records for hash_values...\n"|tee -a $LOG
fi

# drop backups_hash_values indexes to speed up performance
check_worker_error
printf "Dropping indexes from the backups_hash_values table... \n"|tee -a $LOG
start_time=$SECONDS
$psql_path/psql -U $database_user $database_name <<-EOF >> $LOG 2>&1
SET maintenance_work_mem to $maintenance_work_mem;
SET search_path to bp;
DROP INDEX bp.deduplicated_backups_hash_values_index; 
ALTER TABLE bp.backups_hash_values DROP CONSTRAINT backups_hash_values_backup_no_key;
ALTER TABLE bp.backups_hash_values DROP CONSTRAINT backup_no_hash_id_file_offset_unique;
DROP INDEX bp.backups_hash_values_hash_id_index;
EOF
rc=$?
check_elapsed_time "drop_backups_indexes"
check_rc

# Loop through the list of backups to extract the backups_hash_values information
# and load the table as we go
printf "Looping through the backups to extract and load the backups_hash_values data...\n"|tee -a $LOG
printf "Using $metautil -B to extract specific backup data...\n"|tee -a $LOG

migrate_start_time=$SECONDS
while read backupno
do
	check_worker_error
	(( current_num_backups=current_num_backups+1 ))
	DATE=`date +%F-%H:%M:%S`
	printf "${DATE} Running subshell to load backup number: $backupno which is backup $current_num_backups out of $total_num_backups \n" >> $LOG
	if ! ((current_num_backups % 100)); then
		echo -ne "${DATE} Running subshell to load backup number: $backupno which is backup $current_num_backups out of $total_num_backups \r"
		disable_rrc_lines=`wc -l $BPDIR/lists.dir/.disable_rrc|cut -c1`
		sed -ie "${disable_rrc_lines}s/.*/${DATE}: Optimizing backup number: $backupno which is backup $current_num_backups out of $total_num_backups/" $BPDIR/lists.dir/.disable_rrc
	fi
	# Check if backup really has any records associated with it
	# Otherwise count it and move on
	$metautil -c${backupno}|grep -q "^0 references"
	rc=$?
	if (( $rc == 0 )); then
		printf "WARNING: Backup number $backupno had no metadata to migrate...\n" >> $LOG
		(( no_records_backup=no_records_backup+1 ))
		continue
	fi
	if [[ `$PSQL -U $database_user $database_name -tAc "select exists(select 1 from bp.backups where backup_no = $backupno)"` == "f" ]]; then 
		printf "WARNING: Backup number $backupno had no associated backup reference in the backups table...\n" >> $LOG
		(( no_records_backup=no_records_backup+1 ))
		continue
	fi
	(
		start_time=$SECONDS
		# Pipe 0 - metautil
		# Pipe 1 - sed
		# Pipe 2 - awk
		# Pipe 3 - psql 
		$metautil -B${backupno} 2>>$LOG| sed -e '1d' -e '$d'| awk -F' ' '{gsub("c","f",$5);gsub("i","t",$5);if ($4 == 0) print $1 " " $2 " " $3 " " "\\N" " " $5 ; else print $1 " " $2 " " $3 " " $4 " " $5}' | 
		$PSQL -U $database_user $database_name -c"SET work_mem to $work_mem;copy bp.backups_hash_values from STDIN (DELIMITER ' ');" 2>> $LOG
		test_piperc=${PIPESTATUS[*]}
		check_elapsed_time "migrate_backup_${backupno}"
		i=0
                for rc in ${test_piperc[@]}
                do
			# Check the rc for the PSQL piped command
                        if (( $i == 3 )); then
                                if (( $ignore_prompt == 0 )); then
                                        check_worker_rc
                                else
                                        check_worker_rc_backups
                                fi
                        else
                                check_worker_rc
                        fi
                        (( i=i+1 ))
                done
	) &

	workers_process_count=`ps -aef|grep -v grep|grep -w "${script_name}"|wc -l`
	while (( $workers_process_count > $max_workers_process_count ))
	do
		check_worker_error
		workers_process_count=`ps -aef|grep -v grep|grep -w "${script_name}"|wc -l`
		usleep 100000
	done
done < $backup_list_file
wait
(( elapsed_time=SECONDS-migrate_start_time ))
elapsed_time=`echo $(($elapsed_time / 60))`
echo -ne '\n' |tee -a $LOG
printf "migrate_all_backups ran in ${elapsed_time} minutes...\n"|tee -a $LOG
check_worker_error

rm -f $backup_list_file $backup_tbl_list_file $worker_error_file >/dev/null 2>&1

printf "Done loading all hash_values and backup_hash_values records.\n"|tee -a $LOG

# Subshell the creation of the extra indexes for hash_values
(
check_worker_error
printf "Recreating indexes and running analyze for bp.hash_values...\n"|tee -a $LOG
if [[ $bp_version -lt 920 ]]; then
	start_time=$SECONDS
	$PSQL -U $database_user $database_name <<- EOF >> $LOG 2>&1
	SET maintenance_work_mem to $maintenance_work_mem;
	SET search_path to bp;
	CHECKPOINT;
	CREATE INDEX hash_values_sis_offset_index ON bp.hash_values USING btree (sis_offset);
	ALTER TABLE bp.hash_values ADD CONSTRAINT hash_values_hash_type_key UNIQUE (hash_type, hashed_length, hash_value);
	SELECT relname,reltuples::bigint as row_count from pg_class where relname = 'hash_values';
	EOF
else
	$PSQL -U $database_user $database_name <<- EOF >> $LOG 2>&1
	SET maintenance_work_mem to $maintenance_work_mem;
	SET search_path to bp;
	CHECKPOINT;
	CREATE INDEX hash_values_sis_offset_index ON bp.hash_values USING btree (sis_offset);
	ALTER TABLE bp.hash_values ADD CONSTRAINT hash_value_length_type_unique UNIQUE(hash_value,hashed_length,hash_type);
	ANALYZE VERBOSE bp.hash_values;
	SELECT relname,reltuples::bigint as row_count from pg_class where relname = 'hash_values';
	EOF
fi
rc=$?
check_rc
check_elapsed_time "create_hash_indexes"
check_worker_rc
) &

# make sure there weren't any failures with the background processes
(
check_worker_error
printf "Re-creating indexes for bp.backups_hash_values...\n"|tee -a $LOG
start_time=$SECONDS
$PSQL -U $database_user $database_name << EOF >> $LOG 2>&1
SET maintenance_work_mem to $maintenance_work_mem;
SET search_path to bp;
CREATE INDEX backups_hash_values_hash_id_index ON bp.backups_hash_values USING btree (hash_id);
ALTER TABLE bp.backups_hash_values ADD CONSTRAINT backups_hash_values_backup_no_key UNIQUE (backup_no, hash_id, file_offset);
CREATE INDEX deduplicated_backups_hash_values_index ON bp.backups_hash_values USING btree (deduplicated) WHERE (deduplicated IS NULL);
ANALYZE VERBOSE bp.backups_hash_values;
select relname,reltuples::bigint as row_count from pg_class where relname = 'backups_hash_values';
EOF
rc=$?
check_elapsed_time "recreate_backups_indexes"
check_rc
check_worker_rc
) &

# make sure there weren't any failures with the background processes
wait
check_worker_error
if (( $failed_migration == 1 )) || [[ -e $worker_error_file ]]; then
	printf "ERROR: There was an error during the migration.\nPlease review the $LOG file for more details.\n"|tee -a $LOG
	printf "ERROR: Migration failed...\n"|tee -a $LOG
	printf "Exiting script now...\nPlease remember to start all bp processes and cron as necessary...\n"|tee -a $LOG
	printf "RC=14\n" >> $LOG
	DATE=`date +%F%H%M%S`
	printf "$DATE \n" >> $LOG
	clean_up
	exit 14
fi

printf "Verifying number of migrated backups...\n"|tee -a $LOG
start_time=$SECONDS
final_num_backups=`$PSQL -U $database_user $database_name -tq -c"select count(distinct(backup_no)) as unique_backup_count from bp.backups_hash_values;"|head -1|tr -d '[:space:]'`
# Add any number of backup records we found having no records to the count
(( final_num_backups=final_num_backups+no_records_backup ))
check_elapsed_time "verify_backups_count"
if (( $total_num_backups != $final_num_backups )); then
	printf "ERROR: The number of migrated backups does not match.\nPlease review the $LOG file for more details.\n"|tee -a $LOG
	printf "ERROR: There were $total_num_backups out of which only $final_num_backups were migrated\n"|tee -a $LOG
	printf "ERROR: Migration failed...\n"|tee -a $LOG
	printf "Exiting script now...\nPlease remember to start all bp processes and cron as necessary...\n"|tee -a $LOG
	printf "RC=16\n" >> $LOG
	DATE=`date +%F%H%M%S`
	printf "$DATE \n" >> $LOG
	clean_up
	exit 16
fi
printf "We were able to migrate $final_num_backups out of $total_num_backups successfully.\n"|tee -a $LOG
printf "\n" >> $BPDIR/lists.dir/.disable_rrc
printf "`date +%F-%H:%M:%S`: We were able to migrate $final_num_backups out of $total_num_backups successfully.\n" >> $BPDIR/lists.dir/.disable_rrc

# set the sequence object for hash_id and also get a row count for hash_values and backups_hash_values
printf "Setting the maxvalue for the hash_id sequence to the minimum hash_id from the dedupNT metadata...\n"|tee -a $LOG
start_time=$SECONDS
$PSQL -U $database_user $database_name << EOF >> $LOG 2>&1
SET search_path to bp;
\d+ bp.hash_values_hash_id_seq
DO '
DECLARE max_hash_id bigint;
BEGIN 
SELECT (MIN(hash_id)-1)::bigint as hash_id into max_hash_id from bp.hash_values where hash_id > 20000000;
EXECUTE ''ALTER SEQUENCE bp.hash_values_hash_id_seq MAXVALUE '' || max_hash_id; 
END
';
\d+ bp.hash_values_hash_id_seq
EOF
rc=$?
check_elapsed_time "set_sequence"
check_rc

cp -a $BPDIR/bpinit/master.ini $BPDIR/bpinit/master.ini_$DATE >> $LOG 2>&1
printf "Setting MetadataGeneration to 0 in the master.ini file...\n"|tee -a $LOG
$BPDIR/bin/bputil -p "fileDedup" MetadataGeneration "0" $BPDIR/bpinit/master.ini

cp -a /var/opt/unitrends/platform_capabilities /var/opt/unitrends/platform_capabilities_$DATE >> $LOG 2>&1
printf "Taking dedupNT out of the platform_capabilities file...\n"|tee -a $LOG
sed -i '/dedupNT/d' /var/opt/unitrends/platform_capabilities

printf "Renaming the current $meta_dir to ${meta_dir}_old ...\n" | tee -a $LOG
mv $meta_dir ${meta_dir}_old >> $LOG 2>&1

# Update progress file for snmp reporting
new_snmp_status=${migration_folder}/success
printf "Running tar -zcvf /backups/hashMetadata.tar.gz ${meta_dir} in background...\n"|tee -a $LOG
printf "Removing the ${meta_dir}_old folder\n"|tee -a $LOG
(
tar -zcvf /backups/hashMetadata.tar.gz ${meta_dir}_old >> $LOG 2>&1
rm -fR ${meta_dir}_old >> $LOG 2>&1
) &
duration=$SECONDS
duration=`echo $(($duration / 60))`
printf "Migration ran in ${duration} minutes...\n"|tee -a $LOG
printf "`date +%F-%H:%M:%S`: Migration ran in ${duration} minutes successfully.\n" >> $BPDIR/lists.dir/.disable_rrc
printf "END: `date +%F%H%M%S` \n" >> $LOG

printf "Updating Postgres parameters...\n"|tee -a $LOG
$BPDIR/bin/sys_tune >> $LOG 2>&1
if [ $rpm -eq 1 ]; then
  start_services rpm
else
  $BPDIR/bin/setup_postgresql.sh update >> $LOG 2>&1
  start_services 
fi
}

# main script
if [ ! -d ${migration_folder}/ ]; then
        mkdir ${migration_folder}/
fi

# cleanup migration folder
rm -f ${migration_folder}/* > /dev/null 2>&1

# Add progress file to capture status thorugh snmp traps
touch ${snmp_status} 

run_option=""
run_verbiage=" normal "

while getopts rsmfibc opt
do
    case "$opt" in
	m) printf "Will send an email to $email_address on exit...\n" >> $LOG;
	   send_mail=1 ;;
	r) printf "Running with rpm option\n" >> $LOG;
	   run_option="s";
	   force_trunc_prompt=0;
	   rpm=1;;
	s) run_option="s";
	   run_verbiage=" silent ";
	   force_trunc_prompt=1;
	   printf "Bypassing all prompts...\n" >> $LOG;;
	i) printf "Will ignore errors when loading backups_hash_values...\n" >> $LOG;
	   ignore_prompt=1;;
	b) printf "Will load the backups_hash_values data only...\n" >> $LOG;
	   backups_only_prompt=1;;
	f) printf "Truncating any data for hash_values and backups_hash_values...\n" >> $LOG;
	   force_trunc_prompt=1;;
	c) printf "Removing the lock file and motd files from a previous possible failed run...\n" |tee -a $LOG;
	   cleanup_failed_prompt=1;
	   rm -f ${lockFile};;
        *) printf "Invalid parameter passed\n $USAGE"; 
	   exit 5 ;;
    esac
done
shift $((OPTIND-1))

if [ "$run_option" != "s" ]; then
 check_screen=`echo $STY`
 if [[ $check_screen == "" ]]; then
	printf "This script needs to run from a screen session.\nPlease start a screen session and try to run the script again.\n"
	clean_up
	exit 13
 fi
fi
check_psql

printf "Started migration from dedupNT to DB on ${DATE}: \n"|tee -a $LOG
printf "This system has $processor_count cores and $mem_mb MB of RAM...\n">> $LOG

if (( $cleanup_failed_prompt == 1)); then
	#verify that the message in motd is migration related
	rm -f $lockFile ${migration_folder}/started $worker_error_file ${migration_folder}/*failed $BPDIR/lists.dir/.disable_rrc >/dev/null 2>&1
	if [ -s /etc/motd ]; then
		rc=`grep -w dedupNT /etc/motd >/dev/null`
		if [[ $rc -eq 0 ]]; then
			yes|cp -rf /dev/null /etc/motd >> $LOG 2>&1
		else
			printf "The message in motd is not migration related.\n"|tee -a $LOG
			printf "Please verify that the system is stable and try again...\n"|tee -a $LOG
			printf "RC=1\n"|tee -a $LOG
			DATE=`date +%F%H%M%S`
			printf "$DATE \n" >> $LOG
			exit 1
		fi
	fi
fi
#
# If the lock file is older 24 hours give a Warning and exit 
# Maybe a catastrophic event happend
#
if [[ -e ${lockFile} ]]; then
        if test "`find ${lockFile} -mmin +1440`"; then
                printf "Found ${lockFile}.  There is possibly a migration process still running...\n"|tee -a $LOG
		lsof ${lockFile}|tee -a $LOG
		exit 8
        fi
fi

#
#  Make sure only one copy of this script can run at a time.
#
if ! lockfile -r0 "$lockFile"; then
        printf "${script_name}: Can't get the lock\n"|tee -a $LOG
        exit 8
fi

proceed_check
migrate_dedupNT

exit 0
