-- This script is only run on a fresh install!
-- To update or add views, add statements to the update_bpdb.sql script.

set search_path to bp,public;

CREATE OR REPLACE VIEW privileges AS
SELECT user_id, priv_level, customer_id, location_id, CAST(system_id AS integer) FROM
 (SELECT user_id, priv_level, customer_id, null as location_id, null as system_id FROM customer_priv
  UNION ALL
  SELECT user_id, priv_level, null, location_id, null FROM location_priv
 ) AS cl
UNION ALL
  SELECT user_id, priv_level, null, null, system_id FROM system_priv;


CREATE OR REPLACE VIEW ownership AS
SELECT * FROM
  (SELECT system_id, location_id FROM systems) AS s
FULL OUTER JOIN
  (SELECT location_id, customer_id FROM locations) AS l
USING (location_id)
FULL OUTER JOIN
  (SELECT customer_id FROM customers) AS c
USING (customer_id);


CREATE OR REPLACE VIEW inherited_privileges AS
SELECT user_id, priv_level, o.customer_id, o.location_id, o.system_id FROM customer_priv AS p INNER JOIN ownership AS o USING (customer_id)
UNION ALL
SELECT user_id, priv_level, null, o.location_id, o.system_id FROM location_priv AS p INNER JOIN ownership AS o USING (location_id)
UNION ALL
SELECT user_id, priv_level, null, null, system_id FROM system_priv WHERE system_id IS NOT NULL;


CREATE OR REPLACE VIEW viewables AS
SELECT user_id, priv_level, o.*
  FROM privileges AS p
  INNER JOIN ownership AS o
  ON (o.customer_id = p.customer_id OR o.location_id = p.location_id OR o.system_id = p.system_id)
WHERE (priv_level >= 1);

-------------------------------------------------------------------------------------------------------

--------------------------------------------------------------------------------------------------------------
-- datawidth
--------------------------------------------------------------------------------------------------------------
DROP VIEW IF EXISTS bloat_datawidth_bpdb CASCADE;
CREATE OR REPLACE VIEW bloat_datawidth_bpdb AS
SELECT ns.nspname AS schemaname, 
	tbl.oid AS relid, tbl.relname,
  CASE WHEN every(avg_width IS NOT NULL)
  THEN round((SUM((1-null_frac)*avg_width) + MAX(null_frac) * 24)::NUMERIC, 2) ELSE NULL END AS datawidth
  FROM 
  pg_attribute att JOIN pg_class tbl 
  	ON att.attrelid = tbl.oid
  JOIN pg_namespace ns 
  	ON ns.oid = tbl.relnamespace 
  LEFT JOIN pg_stats s
  	ON s.schemaname=ns.nspname 
	AND s.tablename = tbl.relname 
	AND s.attname=att.attname
  WHERE att.attnum > 0 AND tbl.relkind in ('r', 'i') GROUP BY 1,2,3;


--------------------------------------------------------------------------------------------------------------
-- the best way to find bloat is running pgstattuple
-- expectedpages can be off
--------------------------------------------------------------------------------------------------------------
DROP VIEW IF EXISTS bloat_tables_bpdb CASCADE;
CREATE OR REPLACE VIEW bloat_tables_bpdb AS
SELECT s3.schemaname, tblname as relname,
	bw.datawidth,
	reltuples::bigint,tblpages::bigint as relpages,
	(tblpages-est_tblpages_ff)::bigint as expectedpages,
	CASE WHEN tblpages - est_tblpages_ff > 0
	THEN (100 * (tblpages - est_tblpages_ff)/tblpages)::numeric(5,2)
	ELSE 0
	END AS bloatpct
FROM (
         SELECT ceil( reltuples / ( (bs-page_hdr)/tpl_size ) ) + ceil( toasttuples / 4 ) AS
         est_tblpages, ceil( reltuples / ( (bs-page_hdr)*fillfactor/(tpl_size*100) ) ) +
         ceil( toasttuples / 4 ) AS est_tblpages_ff, tblpages, fillfactor, bs, tblid,
         schemaname, tblname, heappages, toastpages,reltuples, is_na
         -- , stattuple.pgstattuple(tblid) AS pst
        FROM (
                 SELECT
                 ( 4 + tpl_hdr_size + tpl_data_size + (2*ma)
                 - CASE WHEN tpl_hdr_size%ma = 0 THEN ma ELSE tpl_hdr_size%ma END
                 - CASE WHEN ceil(tpl_data_size)::int%ma = 0 THEN ma ELSE ceil(tpl_data_size)::int%ma
                 END ) AS tpl_size, bs - page_hdr AS size_per_block, (heappages + toastpages) AS
                 tblpages, heappages, toastpages, reltuples, toasttuples, bs, page_hdr, tblid,
                 schemaname, tblname, fillfactor, is_na
                 FROM (
                         SELECT
                         tbl.oid AS tblid, ns.nspname AS schemaname, tbl.relname AS tblname, tbl.reltuples,
                         tbl.relpages AS heappages, coalesce(toast.relpages, 0) AS toastpages,
                         coalesce(toast.reltuples, 0) AS toasttuples,
                         coalesce(substring(
                         array_to_string(tbl.reloptions, ' ')
                         FROM '%fillfactor=#"__#"%' FOR '#')::smallint, 100) AS fillfactor,
                         current_setting('block_size')::numeric AS bs,
                         CASE WHEN version()~'mingw32' OR version()~'64-bit|x86_64|ppc64|ia64|amd64' THEN 8
                         ELSE 4 END AS ma, 24 AS page_hdr, 23 + CASE WHEN MAX(coalesce(null_frac,0)) > 0 THEN
                         ( 7 + count(*) ) / 8 ELSE 0::int END
                         + CASE WHEN tbl.relhasoids THEN 4 ELSE 0 END AS tpl_hdr_size,
                         sum( (1-coalesce(s.null_frac, 0)) * coalesce(s.avg_width, 1024) ) AS tpl_data_size,
                         bool_or(att.atttypid = 'pg_catalog.name'::regtype) AS is_na
                         FROM pg_attribute AS att
                         JOIN pg_class AS tbl ON att.attrelid = tbl.oid
                         JOIN pg_namespace AS ns ON ns.oid = tbl.relnamespace
                         JOIN pg_stats AS s ON s.schemaname=ns.nspname
                         AND s.tablename = tbl.relname AND s.inherited=false AND s.attname=att.attname
                         LEFT JOIN pg_class AS toast ON tbl.reltoastrelid = toast.oid
                         WHERE att.attnum > 0 AND NOT att.attisdropped
                         AND tbl.relkind = 'r'
                         GROUP BY 1,2,3,4,5,6,7,8,9,10, tbl.relhasoids
                         ORDER BY 2,3
                         ) AS s
        ) AS s2
 ) AS s3 JOIN bp.bloat_datawidth_bpdb bw ON bw.schemaname=s3.schemaname and bw.relid=s3.tblid;

--------------------------------------------------------------------------------------------------------------
-- table size
--------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE VIEW size_tables_bpdb AS
SELECT schemaname, tablename,
  pg_relation_size(schemaname || '.' || tablename) AS size_p,
  pg_total_relation_size(schemaname || '.' || tablename) AS siz,
  pg_total_relation_size(schemaname || '.' || tablename) AS total_size_p,
  pg_total_relation_size(schemaname || '.' || tablename) - pg_relation_size(schemaname || '.' || tablename) AS index_size,
  (100*(pg_total_relation_size(schemaname || '.' || tablename) - pg_relation_size(schemaname || '.' || tablename))) /
  CASE WHEN pg_total_relation_size(schemaname || '.' || tablename) = 0 THEN 1
  ELSE pg_total_relation_size(schemaname || '.' || tablename) END AS index_pct
  FROM pg_tables ORDER BY siz;
