<?php
// defines BP, xml, action, cookie
require 'header.php';

// general list operations.
require 'listsbase.php';
$ListBase = new ListBase();

switch ($action) {

	case "list":
			$bValid = true;
			$selectionType = isset($_GET['list']) ? $_GET['list'] : "all";
			$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();
			if (!isset($_GET['sid']) || $dpuID == $BP->get_local_system_id()) {
				$reports = $BP->get_reports();
			} else {
				$reports = $BP->get_reports($dpuID);
			}
			if ($reports === false) {
				$bValid = false;
			} else {
				$xml->push("root");
				foreach ($reports as $report) {
					$attributes = array("Name" => $report['name'], "ID" => $report['report_id']);
					if (array_key_exists('description', $report)) {
						$attributes = array_merge($attributes, array("Description" => $report['description']));
					}
					if (array_key_exists('based_on', $report)) {
						$attributes = array_merge($attributes, array("BasedOn" => $report['based_on']));
					}
					$xml->push("Report", $attributes);
					// Add the fields to the xml content.
					if (array_key_exists('fields', $report)) {
						$xml->content($report['fields']);
					}
					$xml->pop();
				}
				$xml->pop(); //root
			}
			if (!$bValid) {
				$xml->clear(false);
				$BP->buildResult($xml, false);
			}
			echo($xml->getXml());
			break;
					
	case "save":
			// save a list of name/value pairs as a specified type and name.
			if (isset($_GET['name'])) {
				$name = $_GET['name'];
				if (isset($_GET['description'])) {
					$description = $_GET['description'];
					$basedon = $_GET['basedon'];
					
					if (isset($_GET['nvp'])) {
						$nvpXML = buildNVPXML($_GET['nvp']);
						if ($nvpXML !== false) {
							$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();
							$reportInfo = array('name' => $name,
												'description' => $description,
												'based_on' => $basedon,
												'fields' => $nvpXML);
							//update requires an id
							if ( isset($_GET['id']) ) {
								$reportInfo['report_id'] = (int)$_GET['id'];
								if (isset($_REQUEST['OtherUsersFieldsXML']) && isset($_REQUEST['HasReportsForThisUser']) ) {
									if ($_REQUEST['HasReportsForThisUser'] == "true") {
										$nvpXML .= $_REQUEST['OtherUsersFieldsXML'];
									} else {
										$nvpXML = $_REQUEST['OtherUsersFieldsXML'];
									}
									$reportInfo['fields'] = $nvpXML;			
								}
							}
							$BP->buildResult($xml, $BP->save_report_info($reportInfo, $dpuID));
						} else {
							$BP->buildResult($xml, false, "error: reports.php: error parsing field definitions.");
						}
					} else {
						$BP->buildResult($xml, false, "error: nvp.php: no fields specified.");
					}
				} else {
					$BP->buildResult($xml, false, "error: no report description specified.");
				}
			} else {
				$BP->buildResult($xml, false, "error: no report name specified.");
			}
			echo($xml->getXml());
			break;
				
	case "delete":
			$bValid = true;
			$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();
			if (($id = $ListBase->getID($BP, 'get_reports', $dpuID)) == -1) {
				$BP->buildResult($xml, false, "Error: cannot find report for deletion.");
			} else {
				$BP->buildResult($xml, $BP->delete_report($id, $dpuID));
			}
			echo($xml->getXml());
			break;
				
	case "file":
			if (isset($_GET['report']) && isset($_REQUEST['contents'])) {
				$reportType = $_GET['report'];
				$contents = $_REQUEST['contents'];
			} else {
				$BP->buildResult($xml, false, "error: report type, and content needed for saving file.");
				echo($xml->getXml());
				break;
			}
			$baseName = isset($_GET['name']) ? $_GET['name'] : 'report';
			$reportDirectory = $BP->get_ini_value("Location Information", "Reports-Dir");
			if ($reportDirectory === false) {
				// Since we are not erroring out, log in the error log.
				// Use the default value /usr/bp/logs.dir.
				global $Log;
				$message = $BP->getError() . " - Error retrieving ini field: Location Information, Report-Dir, using default (/usr/bp/reports.dir).";
				$Log->writeError($message, true);
				$reportDirectory = "/usr/bp/reports.dir";
			}
			$bValid = valid_param($contents);
			if (!$bValid) {
				$BP->buildResult($xml, false, "error: invalid report contents.");
				echo($xml->getXml());
				break;
			}
			$bValid = valid_param($reportType);
			if (!$bValid) {
				$BP->buildResult($xml, false, "error: invalid report type.");
				echo($xml->getXml());
				break;
			}
			$bValid = valid_dir($reportDirectory);
			if (!$bValid) {
				$BP->buildResult($xml, false, "error: invalid report directory.");
				echo($xml->getXml());
				break;
			}
			$bValid = valid_dir($basename);
			if (!$bValid) {
				$BP->buildResult($xml, false, "error: invalid report filename.");
				echo($xml->getXml());
				break;
			}
			$bValid = valid_type($reportType);
			if (!$bValid) {
				$BP->buildResult($xml, false, "error: invalid report filename.");
				echo($xml->getXml());
				break;
			}
			$fileName = createReportName($baseName, $reportDirectory, $reportType);
			$bSuccessful = saveReport($fileName, $contents);
			if ($bSuccessful === true) {
				$xml->push("root");
				$xml->element("ReportFile", $fileName);
				$xml->pop();
			} else {
				$errorString = "Error saving report file '" . $fileName . "'.";
				$BP->buildResult($xml, false, $errorString);
			}
			echo($xml->getXml());
			break;
			
}

//
// This function returns a name of a report (the CSV file) based on the type of report and date/time.
//
function createReportName($baseName, $directory, $type)
{
	$sName = $directory . '/' . $baseName;
	$timestamp = time();
	$date = date('mdy-His', $timestamp);
	
	$sName .= $date . '.' . $type;
	
	return $sName;
}

function valid_param($value)
{
        $rval = true;
	if (strpos($value, '`') !== FALSE) {
		$rval = false;
	}
	if ($rval == false) {
		global $Log;
		$msg = date('Y-m-d H:i:s')." reports.php: detected invalid parameter\n";
		$Log->writeError($message, true);
	}
	return $rval;
}

function valid_dir($value)
{
	$rval = true;
	if (strpos($value, '`') !== FALSE) {
		$rval = false;
	}
	if (strpos($value, '$') !== FALSE) {
		$rval = false;
	}
	if (strpos($value, '/var/www') !== FALSE) {
		$rval = false;
	}
	if (strpos($value, 'php') !== FALSE) {
		$rval = false;
	}
	if (strpos($value, '..') !== FALSE) {
		$rval = false;
	}
	if ($rval == false) {
		global $Log;
		$msg = date('Y-m-d H:i:s')." reports.php: detected invalid directory parameter\n";
		$Log->writeError($message, true);
	}
	return $rval;
}

function valid_type($value)
{
	$rval = false;
	if (strpos($value, '`') !== FALSE) {
		return $rval;
	}
	if (strpos($value, '$') !== FALSE) {
		return $rval;
	}
	// use a whitelist of expected report data types
	$types = array("csv", "xls", "txt", "pdf", "htm", "xml", "doc");
	foreach ( $types as $type ) {
		if (strpos($value, $type) !== FALSE) {
			$rval = true;
			break;
		}
	}
	if ($rval == false) {
		global $Log;
		$msg = date('Y-m-d H:i:s')." reports.php: invalid file type $value\n";
		$Log->writeError($message, true);
	}
	return $rval;
}

//
// This function saves the contents of the report to the file and returns
// true if successful and false if not.
//
function saveReport($file, $contents)
{
	$bSuccessful = false;
	$fp = fopen($file, 'w+');
	if ($fp !== false) {
		fwrite($fp, $contents);
		fclose($fp);
		$bSuccessful = true;
	}
	return $bSuccessful;
}

//
// This function builds an XML string for the fields from a set of name/value pairs (nvpString).
// Returns the XML content as a string.
//
function buildNVPXML($nvpString)
{
	$xmlReports = new ria_xmlWriter(false, false);
	global $stringSeparator;

	// nvp string will be in the form <name><separator><value>,<name><separator><value>, ...
	
	// strip quotes
	$nvpString = trim($nvpString, ",");
	$nameValuePairs = explode(",", $nvpString);

	foreach ($nameValuePairs as $nameValuePair) {
		//unique separator string.  Also in Flex user services file.
		$pieces = explode($stringSeparator, $nameValuePair);
		if ($pieces[0] != "") {
			$name = $pieces[0];
			$value = $pieces[1];
			$userName = "";
			if ( count($pieces) > 2 ) {
				$userName = $pieces[2];		
			}
			$xmlReports->emptyelement("Field", array("Name" => $name, "Value" => $value, "ReportUserName" => $userName));
		}		
	}
	return $xmlReports->getXml();
}

$BP->destroyCookie();
?>
